//-----------------------------LICENSE NOTICE------------------------------------
//  This file is part of Bitume 
//  Copyright (C) 2017 Arnaud Bouche (Arnaud6128)
//
//  This program is free software: you can redistribute it and/or modify
//  it under the terms of the GNU Lesser General Public License as published by
//  the Free Software Foundation, either version 3 of the License, or
//  (at your option) any later version.
//
//  This program is distributed in the hope that it will be useful,
//  but WITHOUT ANY WARRANTY; without even the implied warranty of
//  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
//  GNU Lesser General Public License for more details.
//
//  You should have received a copy of the GNU Lesser General Public License
//  along with this program.  If not, see <http://www.gnu.org/licenses/>.
//-------------------------------------------------------------------------------

/*************** System includes ***************/
#include <cpctelera.h>
#include "wincpctelera_helper.h"

/*************** Game includes ****************/
#include "structures.h"
#include "sound.h"
#include "video.h"
#include "road.h"
#include "player.h"
#include "cars.h"
#include "fontsHelper.h"
#include "move.h"
#include "gun.h"
#include "toolkit.h"
#include "mines.h"
#include "explosions.h"
#include "objects.h"
#include "board.h"

/*************** Sprites includes *************/
#include ".\gen_src\car_en1.h"
#include ".\gen_src\car_en2.h"
#include ".\gen_src\car_en3.h"
#include ".\gen_src\car_en4.h"
#include ".\gen_src\car_en5.h"
#include ".\gen_src\car_en6.h"
#include ".\gen_src\car_en7.h"
#include ".\gen_src\car_en8.h"
#include ".\gen_src\car_en9.h"
#include ".\gen_src\car_en11.h"
#include ".\gen_src\barrel.h"

#include ".\gen_src\shoot.h"
#include ".\gen_src\shoot_double.h"

/*************** Local defines ****************/
#define LEFT_POS			-20
#define RIGHT_POS			100
#define MAX_RIGHT_POS		127

/*************** Cars informations ************/
const SCarGraphics gCarStyleset[] =
{
	/*GRAPH_CAR_1*/		{ G_CAR_EN1_W,	G_CAR_EN1_H,	g_car_en1 },
	/*GRAPH_CAR_2*/		{ G_CAR_EN2_W,	G_CAR_EN2_H,	g_car_en2 },
	/*GRAPH_CAR_3*/		{ G_CAR_EN3_W,	G_CAR_EN3_H,	g_car_en3 },
	/*GRAPH_CAR_4*/		{ G_CAR_EN4_W,	G_CAR_EN4_H,	g_car_en4 },
	/*GRAPH_TRUCK_5*/	{ G_CAR_EN5_W,	G_CAR_EN5_H,	g_car_en5 },
	/*GRAPH_CAR_6*/		{ G_CAR_EN6_W,	G_CAR_EN6_H,	g_car_en6 },
	/*GRAPH_CAR_7*/		{ G_CAR_EN7_W,	G_CAR_EN7_H,	g_car_en7 },
	/*GRAPH_CAR_8*/		{ G_CAR_EN8_W,	G_CAR_EN8_H,	g_car_en8 },
	/*GRAPH_MBIKE_9*/	{ G_CAR_EN9_W,	G_CAR_EN9_H,	g_car_en9 },
	/*GRAPH_CAR_11*/	{ G_CAR_EN11_W,	G_CAR_EN11_H,	g_car_en11 },
	/*GRAPH_BARREL*/	{ G_BARREL_W,	G_BARREL_H,		g_barrel },
	/*GRAPH_NO */		{ 0,			0,				NULL }
}; 

#define CAR_EN1_POSY	(G_CAR_EN1_H-G_SHOOT_0_H)/2
#define CAR_EN11_POSY	(G_CAR_EN11_H-G_SHOOT_0_H)/2
#define CAR_EN9_POSY	(G_CAR_EN9_H-G_SHOOT_0_H)/2+1

const SCarInfo gCarInfos[] =
{
	/**			Armor/Protection,	Weapon,						WeaponPosY		Speed/Score,	Graphics,		Move */
	/*EN_CAR_1*/		{ 0x31,		GUN_DOUBLE_RIGHT,			2,					0x1A,		GRAPH_CAR_1,	MoveEnterFromRightToRear },		// RIGHT
	/*EN_BIKE_2*/		{ 0x20,		GUN_SIMPLE_LEFT,			10,					0x1A,		GRAPH_CAR_2,	MoveEnterFromRightToFront },	// LEFT
	/*EN_CAR_3_UP*/		{ 0x41,		GUN_DOWN,					G_CAR_EN3_H,	0x18,		GRAPH_CAR_3,	MoveLinearToRight },			// RIGHT
	/*EN_CAR_3_DO*/		{ 0x41,		GUN_UP,						-G_SHOOT_0_H,		0x18,		GRAPH_CAR_3,	MoveLinearToRight },			// RIGHT
	/*EN_CAR_4*/		{ 0x21,		MINE,						0,					0x1A,		GRAPH_CAR_4,	MoveOverTakeFromLeft },			// LEFT
	/*EN_TRUCK_5*/		{ 0x52,		MISSILE,					0,					0x1F,		GRAPH_TRUCK_5,	MoveEnterFromRightToFront },	// RIGHT
	/*EN_CAR_6*/		{ 0x41,		GUN_DOUBLE_RIGHT_ANTIARMOR,	2,					0x1F,		GRAPH_CAR_6,	MoveEnterFromLeft },			// LEFT
	/*EN_CAR_7*/		{ 0x10,		GUN_NOTHING,				0,					0x15,		GRAPH_CAR_7,	MoveToCollide },				// LEFT
	/*EN_CAR_8*/		{ 0x31,		GUN_DOUBLE_LEFT,			2,					0x19,		GRAPH_CAR_8,	MoveEnterFromRightToFront },	// RIGHT
	/*EN_MBIKE_9*/		{ 0x20,		GUN_SIMPLE_RIGHT,			CAR_EN9_POSY,		0x15,		GRAPH_MBIKE_9,	MoveEnterFromLeft },			// LEFT
	/*EN_MBIKE_9_R*/	{ 0x20,		GUN_SIMPLE_RIGHT,			CAR_EN9_POSY,		0x15,		GRAPH_MBIKE_9,	MoveEnterFromRightToRear },		// RIGHT
	/*EN_MBIKE_9_N*/	{ 0x20,		NAILS,						0,					0x15,		GRAPH_MBIKE_9,	MoveEnterFromRightToFront },	// RIGHT
	/*EN_CAR_11*/		{ 0x42,		GUN_HEAVY_LEFT,				CAR_EN11_POSY,		0x19,		GRAPH_CAR_11,	MoveEnterFromRightToFront },	// RIGHT
	/*EN_BARREL*/		{ 0x30,		GUN_NOTHING,				0,					0x00,		GRAPH_BARREL,	MoveCarDestroyed },				// RIGHT
	/*EN_MINE_FIELD*/	{ 0x00,		GUN_NOTHING,				0,					0x00,		GRAPH_NO,		MoveCarDestroyed }				// RIGHT
};		

/*****************************************************/
/*													 */ 
/*	Set gun dimensions								 */
/*												     */
/*****************************************************/
void SetGunSize(SCar* car)
{
	if (car->_gun & GUN_DOUBLE)
	{
		car->_gunSprite._cx = G_SHOOT_DOUBLE_0_W;
		car->_gunSprite._cy = G_SHOOT_DOUBLE_0_H;
	}
	else if (car->_gun & (GUN_SIMPLE|GUN_HEAVY_GUN))
	{
		car->_gunSprite._cx = G_SHOOT_0_W;
		car->_gunSprite._cy = G_SHOOT_0_H;
	}
	else
	{
		car->_gunSprite._cx = G_SHOOT_4_W;
		car->_gunSprite._cy = G_SHOOT_4_H;
	}
}

/*****************************************************/
/*													 */
/*	Delete all cars									 */
/*												     */
/*****************************************************/
void DeleteCars()
{
	cpct_memset(&gRoad._cars, 0, sizeof(SCars));
}

/*****************************************************/
/*													 */
/*	Get start position X							 */
/*												     */
/*****************************************************/
i8 GetStartPositionX(SCar* newCar)
{
	if (newCar->_type == EN_MINE_FIELD)
		return MAX_RIGHT_POS;
	else
	{
		u8 random = Rand()*2;
		if (newCar->Move == MoveEnterFromRightToFront || newCar->Move == MoveEnterFromRightToRear || newCar->Move == MoveCarDestroyed)
			return RIGHT_POS + random;

		return LEFT_POS - random;
	}
}

/*****************************************************/
/*													 */
/*	Get start position Y							 */
/*												     */
/*****************************************************/
i8 GetStartPositionY(u8 posY, u8 cy)
{
	if (posY == START_RAND_CAR)
		return GetRandPositionY();

	return posY - (posY == START_DOWN_CAR ? cy : 0);
}

/*****************************************************/
/*													 */
/*	Add Car											 */
/*												     */
/*****************************************************/
void AddCar(u8 posY, u8 carType)
{
	if (gRoad._cars._nbCars < NB_CARS_MAX)
	{
		u8* weapon;
		SCar* newCar = &gRoad._cars._car[gRoad._cars._nbCars++];		
		SRoadObject* newRoadObj = &newCar->_roadObj;

		const SCarInfo* carInfo = &gCarInfos[carType];
		const SCarGraphics* carGraphics = &gCarStyleset[carInfo->_graphics];
		
		cpct_memset(newCar, 0, sizeof(SCar));
				
		weapon = (carInfo->_weapon > GUNS) ? &newCar->_weapon : &newCar->_gun;
		*weapon = carInfo->_weapon;		

		newCar->_type		= carType;
		newCar->_armor		= carInfo->_armorProtect & 0xF0;
		newCar->_protection = carInfo->_armorProtect & 0x0F;
		newCar->_speed		= (carInfo->_speedScore & 0xF0) >> 4;
		newCar->Move		= carInfo->_move;
		newCar->_flames		= NO_FLAMES;
		newCar->_nbShoot	= (carInfo->_weapon & GUN_HEAVY_GUN) ? 100 : Rand() + 4;
		newCar->_status		= STATUS_ENTER;
		
		if (carInfo->_weapon <= GUNS)
		{
			SetGunSize(newCar);
		}
	
		newRoadObj->_spriteInfo._sprite = carGraphics->_sprite;
		newRoadObj->_spriteInfo._cx = carGraphics->_cx;
		newRoadObj->_spriteInfo._cy = carGraphics->_cy;

		SetCoord(newRoadObj, GetStartPositionX(newCar), GetStartPositionY(posY, carGraphics->_cy));

		UpdateCarAndGunPos(newCar);
	}
}

/*****************************************************/
/*													 */
/*	Delete Car										 */
/*												     */
/*****************************************************/
void DeleteCar(SCar* carToDel)
{
	SCar* lastCar = &gRoad._cars._car[--gRoad._cars._nbCars];

	if (carToDel != lastCar)
		cpct_memcpy(carToDel, lastCar, sizeof(SCar));

	lastCar->_status = STATUS_NOTHING;
}

/*****************************************************/
/*													 */
/*	Draw Cars										 */
/*												     */
/*****************************************************/
void DrawCars() 
{
	SCar* itCar = gRoad._cars._car;
	
	u8 i;
	for (i = 0; i < gRoad._cars._nbCars; i++)
	{
		SSpriteInfo* sprite = &itCar->_roadObj._spriteInfo;

		DrawClippedSprite(sprite->_sprite, sprite->_x, sprite->_y, sprite->_cx, sprite->_cy, SPRITE_OPAQUE, TRUE);

		if (itCar->_status & STATUS_FLAMES)
			DrawFlames(itCar);
		else
		if (itCar->_status & STATUS_SHOOT)
			DrawShoot(itCar);

		itCar++;
	}
}

/*****************************************************/
/*													 */
/*	Draw background Cars							 */
/*												     */
/*****************************************************/
void ClearCars()
{
	SCar* itCar = gRoad._cars._car;

	u8 i;
	for (i = 0; i < gRoad._cars._nbCars; i++)
	{
		SSpriteInfo* sprite = &itCar->_roadObj._spriteInfo;
		SSpriteInfo* gunSprite = &itCar->_gunSprite;

		StoreCurrentPos(sprite);
		StoreCurrentPos(gunSprite);

		itCar->Move(itCar);

		UpdateCarAndGunPos(itCar);

		DrawRoadObjectBackground(sprite);
		DrawRoadObjectBackground(gunSprite);

		if (itCar->_status & STATUS_FLAMES)
			ClearFlamesBackground(itCar);

		itCar++;
	}
}

/*****************************************************/
/*													 */
/*	Update Cars										 */
/*												     */
/*****************************************************/
void UpdateCars()
{	
	static u8 sWait = 0;

	SCar* itCar = gRoad._cars._car;

	u8 i;
	for (i = 0; i < gRoad._cars._nbCars; i++)
	{
		SSpriteInfo* sprite = &itCar->_roadObj._spriteInfo;
		u8 type = itCar->_type;
		BOOL isOutByRightPos = GetPosRightX(sprite) < 0;

		if (itCar->_status & STATUS_FLAMES)
			itCar->_flames++;

		if (((type == EN_MINE_FIELD || type == EN_BARREL) && isOutByRightPos) || !(itCar->_status & STATUS_ENTER) && (sprite->_x > SCREEN_CX_BYTE + 10 || isOutByRightPos))
			DeleteCar(itCar);

		itCar++;
	}
	
	if (GET_PLAYER_CAR._status & STATUS_ACTIVE)
	{
		if (++sWait > 50)
		{
			sWait = 0;

			if (IsRoundFinished())
			{
				gRoad._curRound++;

				if (IsLevelFinished())
				{
					GET_PLAYER_CAR._status = STATUS_END_LEVEL;
					gRoad._curLevel++;
					gRoad._curRound = 0;					
				}
				else
					StartRound();
			}
		}
	}
}

/*****************************************************/
/*													 */
/*	Car destroyed									 */
/*												     */
/*****************************************************/
void CarDestroyed(SCar* car)
{
	u8 carType = car->_type;
	SSpriteInfo* sprite = &car->_roadObj._spriteInfo;

	if (carType == EN_BARREL)
	{
		AddExplosionObject(sprite);
		DeleteCar(car);
	}
	else
	{
		car->_flames = Rand() / 4;
		car->_status = STATUS_FLAMES;
		car->_roadObj._counter = 0;

		/* Player is destroyed **/
		if (carType == PLAYER_CAR)
			PlayerCarDestroyed();
		else
		{
			u8 money = gCarInfos[carType]._speedScore & 0x0F;
			ChangeMove(car, MoveCarDestroyed);
		
			GET_PLAYER._money += money*10;

			UpdateInfoScore();
			AddRoadBonus(money);

			if (carType == EN_CAR_3_UP || carType == EN_CAR_3_DO)
				AddObject(GetBonusType(), sprite->_x - 4, sprite->_y + 8);
		}
	}

	PlaySFX(FX_EXPLOSION, FX_EXPLOSION_CHANNEL);
}

/*****************************************************/
/*													 */
/*	Damage car										 */
/*												     */
/*****************************************************/
void DamageCar(SCar* car, u8 damage)
{
  	if (car->_status != STATUS_FLAMES && damage > car->_protection)
	{
		DecrValue(&car->_armor, damage - car->_protection);
		if (car->_armor == 0)
			CarDestroyed(car);
		else 
		if (car->_type == PLAYER_CAR)
		{
			UpdateInfoArmor();
			if (GET_PLAYER._bonus._id == BONUS_SHIELD)
				PlayerCarDecrBonus(DECR_BONUS_SHIELD);
		}
	}
}
