//-----------------------------LICENSE NOTICE------------------------------------
//  This file is part of Bitume 
//  Copyright (C) 2017 Arnaud Bouche (Arnaud6128)
//
//  This program is free software: you can redistribute it and/or modify
//  it under the terms of the GNU Lesser General Public License as published by
//  the Free Software Foundation, either version 3 of the License, or
//  (at your option) any later version.
//
//  This program is distributed in the hope that it will be useful,
//  but WITHOUT ANY WARRANTY; without even the implied warranty of
//  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
//  GNU Lesser General Public License for more details.
//
//  You should have received a copy of the GNU Lesser General Public License
//  along with this program.  If not, see <http://www.gnu.org/licenses/>.
//-------------------------------------------------------------------------------

#include <cpctelera.h>

#define BOOL				u8
#define TRUE				(BOOL)1
#define FALSE				(BOOL)0

#define VIDEO_MEM			1
#define BUFFER_MEM			0

#define STOP 				while(1){}

#define NB_BUFFER_VIDEO		2
#define SCREEN_SIZE			0x4000
#define SCREEN_ADDRESS		0x8000
#define SCREEN_BUFF			(u8*)SCREEN_ADDRESS

#define MASKTABLESIZE		0x100
#define STACKLOC			(SCREEN_ADDRESS-MASKTABLESIZE-1)
#define STACK_LIMIT			(STACKLOC - 0x64)
#define MASKTABLE			(SCREEN_ADDRESS-MASKTABLESIZE)
#define FREE_AREA_MEM		0x9000

#define SCREEN_CX_BYTE		(i8)(160/2)
#define SCREEN_CY_BYTE		(u8)200

#define ROAD_BORDER_CX		20
#define ROAD_BORDER_CY		6

#define BOARD_CX			40
#define BOARD_CY			37

#define ROAD_BORDER_UP		BOARD_CY
#define ROAD_BORDER_DOWN	(u8)(SCREEN_CY_BYTE - ROAD_BORDER_CY)
#define SIZE_ROAD_SPRITE	ROAD_BORDER_CX*ROAD_BORDER_CY

#define TIME_TO_DEL			(i8)-1
#define IMMEDIATE_DEL		(i8)-3
#define NO_SET_I8			(i8)-1

#define NO_FLAMES			255
#define NB_ARMOR_COLORS		6
#define SPEED_SCROLL		1
#define BONUS_COUNTER		120

#define GUN_START_COUNTER	(i8)4
#define GUN_END_COUNTER		(i8)1

#define	CLEAR_SPRITES		TRUE
#define	DRAW_SPRITES		FALSE

#define CX(X) 		(sizeof(X)/sizeof(u8*))

typedef void(*TInterrupt)(void);

enum
{
	SPRITE_OPAQUE = 0,
	SPRITE_MASKED
};

enum
{
	SCREEN_BUFFER = 0x01,
	BACK_BUFFER = 0x02,
	BOTH_BUFFER = 0x04
};

enum
{
	STATUS_NOTHING = 0x00,
	STATUS_ACTIVE = 0x01,
	STATUS_SHOOT = 0x02,
	STATUS_DROP_MINE = 0x04,
	STATUS_FLAMES = 0x08,
	STATUS_ENTER = 0x10,
	STATUS_EXIT = 0x20,
	STATUS_END_LEVEL = 0x40,
	STATUS_DELETE = 0x80
};

enum
{
	VKEY_NOTHING = 0x00,
	VKEY_RIGHT = 0x01,
	VKEY_LEFT = 0x02,
	VKEY_UP = 0x04,
	VKEY_DOWN = 0x08,
	VKEY_ACTION = 0x10,
	VKEY_SELECTION = 0x20,
	VKEY_ESCAPE = 0x40
};

enum
{
	KEY_FOR_UP = 0,
	KEY_FOR_DOWN,
	KEY_FOR_LEFT,
	KEY_FOR_RIGHT,
	KEY_FOR_FIRE,
	KEY_FOR_WEAPON,
	NB_KEY_FOR
};

enum
{
	NOTHING,
	MENU = 0x01,
	GAME = 0x02,
	WIN = 0x04,
	BLACK_SCREEN = 0x08,
	EXIT = 0x10
};

enum
{
	UP = 0,
	RIGHT,
	LEFT,
	DOWN,
	NONE
};

enum
{
	NO_WEAPON = 0,
	GUNS = 0xB0,
	MINE = 0xB1,
	NAILS,
	MISSILE,
	HEAVY_GUN,
	ROAD_BONUS_LIFE,
	ROAD_BONUS_MISSILE,
	ROAD_BONUS_MINE,
	ROAD_BONUS_SHIELD,
	ROAD_BONUS_TEMPORARY = ROAD_BONUS_SHIELD,
	ROAD_BONUS_AMMO_ANTIARMOR,
	ROAD_BONUS_REPAIR,
	ROAD_BONUS_SPEED,
	ROAD_BONUS_NUM = ROAD_BONUS_SPEED
};

enum
{
	BONUS_SHIELD,
	BONUS_AMMO,
	BONUS_REPAIR,
	BONUS_SPEED,
	BONUS_NOTHING
};

enum
{
	GRAPH_CAR_1 = 0,
	GRAPH_CAR_2,
	GRAPH_CAR_3,
	GRAPH_CAR_4,
	GRAPH_TRUCK_5,
	GRAPH_CAR_6,
	GRAPH_CAR_7,
	GRAPH_CAR_8,
	GRAPH_MBIKE_9,
	GRAPH_CAR_11,
	GRAPH_BARREL,
	GRAPH_NO
};

enum
{
	EN_CAR_1 = 0,
	EN_BIKE_2,
	EN_CAR_3_UP,
	EN_CAR_3_DO,
	EN_CAR_4,
	EN_TRUCK_5,
	EN_CAR_6,
	EN_CAR_7,
	EN_CAR_8,
	EN_MBIKE_9,
	EN_MBIKE_9_R,
	EN_MBIKE_9_N,	
	EN_CAR_11,
	EN_BARREL,
	EN_MINE_FIELD,
	EN_MINE,
	EN_NAILS,
	EN_BONUS_LIFE,
	EN_BONUS_REPAIR,
	EN_BONUS_ARMOR,
	EN_BONUS_MISSILE,
	EN_BONUS_MINE,
	EN_BONUS_AMMO_ANTIARMOR,
	PLAYER_CAR = 254,
	EN_NO = 255
};

enum
{
	WEAPON_GUN = 0,
	WEAPON_MINE,
	WEAPON_MISSILE,
	WEAPON_NB
};

enum
{
	IMPACT_DAMAGES = 1,
	FIRE_DAMAGES = 2,
	SHOOT_DAMAGES = 3,
	NAILS_DAMAGES = 8,
	EXPLOSION_DAMAGES = 3,
	EXPLOSION_CAR = 30,
	MISSILE_DAMAGES = 40
};

