//-----------------------------LICENSE NOTICE------------------------------------
//  This file is part of Bitume 
//  Copyright (C) 2017 Arnaud Bouche (Arnaud6128)
//
//  This program is free software: you can redistribute it and/or modify
//  it under the terms of the GNU Lesser General Public License as published by
//  the Free Software Foundation, either version 3 of the License, or
//  (at your option) any later version.
//
//  This program is distributed in the hope that it will be useful,
//  but WITHOUT ANY WARRANTY; without even the implied warranty of
//  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
//  GNU Lesser General Public License for more details.
//
//  You should have received a copy of the GNU Lesser General Public License
//  along with this program.  If not, see <http://www.gnu.org/licenses/>.
//-------------------------------------------------------------------------------

/*************** System includes ***************/
#include <stdio.h>
#include "wincpctelera_helper.h"

/*************** Game includes ****************/
#include "structures.h"
#include "video.h"
#include "road.h"
#include "cars.h"
#include "player.h"
#include "explosions.h"
#include "sound.h"

/*************** External variables ***********/
extern SRoad gRoad;

/*****************************************************/
/*													 */
/*	Add explosion									 */
/*												     */
/*****************************************************/
void AddExplosion(u8 posX, u8 posY)
{	
	if (gRoad._explosions._nbExplosions < NB_EXPLOSIONS_MAX)
	{
		SRoadObject* newExplosion = &gRoad._explosions._explosion[gRoad._explosions._nbExplosions++];
		newExplosion->_counter = 0;

		SetCoord(newExplosion, posX, posY);
		PlaySFX(FX_EXPLOSION, FX_EXPLOSION_CHANNEL);
	}
}

/*****************************************************/
/*													 */
/*	Add explosion on object							 */
/*												     */
/*****************************************************/
void AddExplosionObject(SSpriteInfo* sprite)
{
	AddExplosion(sprite->_x, GetLimitY(sprite->_y, EXPLOSION_CY));
}

/*****************************************************/
/*													 */
/*	Draw explosions									 */
/*												     */
/*****************************************************/
void DrawExplosions()
{
	static const u8 sExplosion[] = { 0, 1, 2, 2, 2, 1, 0, 3 };
	u8* nbExplosions = &gRoad._explosions._nbExplosions;

	if (*nbExplosions > 0)
	{
		SRoadObject* itExplosion = gRoad._explosions._explosion;

		u8 i;
		for (i = 0; i < *nbExplosions; i++)
		{
			if (itExplosion->_counter > TIME_TO_DEL)
			{
				SSpriteInfo* sprite = &itExplosion->_spriteInfo;
				i8* counter = &itExplosion->_counter;
				sprite->_x -= SPEED_SCROLL;

				if (*counter < 6)
				{
					SCar* collidedCar = TestCarCollision(sprite);
					if (collidedCar != NULL)
						DamageCar(collidedCar, EXPLOSION_DAMAGES);
				}

				if (*counter < sizeof(sExplosion) - 1)
					(*counter)++;

				if (sprite->_x + EXPLOSION_CX < 0)
					*counter = IMMEDIATE_DEL;
				else
					DrawClippedSprite(gFire_tileset[sExplosion[*counter]], sprite->_x, sprite->_y, EXPLOSION_CX, EXPLOSION_CY, SPRITE_OPAQUE, TRUE);
			}
			itExplosion++;
		}

		DeleteRoadObjects(gRoad._explosions._explosion, nbExplosions);
	}
}