//-----------------------------LICENSE NOTICE------------------------------------
//  This file is part of Bitume 
//  Copyright (C) 2017 Arnaud Bouche (Arnaud6128)
//
//  This program is free software: you can redistribute it and/or modify
//  it under the terms of the GNU Lesser General Public License as published by
//  the Free Software Foundation, either version 3 of the License, or
//  (at your option) any later version.
//
//  This program is distributed in the hope that it will be useful,
//  but WITHOUT ANY WARRANTY; without even the implied warranty of
//  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
//  GNU Lesser General Public License for more details.
//
//  You should have received a copy of the GNU Lesser General Public License
//  along with this program.  If not, see <http://www.gnu.org/licenses/>.
//-------------------------------------------------------------------------------

/*************** System includes ***************/
#include <cpctelera.h>
#include <stdio.h>

/*************** Game includes ****************/
#include "structures.h"
#include "toolkit.h"
#include "video.h"

/*************** Sprites includes *************/
#include ".\gen_src\fonts.h"
#include ".\gen_src\fonts_in_game.h"

/*************** Static variables *************/
static const BOOL sInGameFont = 0;
static const u8 sBuffer = 0;

/*************** Defines *********************/
#define NUMBER_FONTS	12

#define FONT_DOT		10
#define FONT_MINUS		11
#define FONT_SPACE		38

#define FONTS_GAME_CX	2
#define FONTS_GAME_CY	7
#define OFFS_FONT_GAME	FONTS_GAME_CX*FONTS_GAME_CY

#define FONTS_CX		2
#define FONTS_CY		8
#define OFFS_FONT_MENU	FONTS_CX*FONTS_CY

/*****************************************************/
/*													 */
/*	Set font type							 		 */
/*												     */
/*****************************************************/
void SetGameFont(BOOL fontInGame)
{
	*(u8*)&sInGameFont = fontInGame;
}

/*****************************************************/
/*													 */
/*	Set buffer								 		 */
/*												     */
/*****************************************************/
void SetBufferFont(u8 bufferSel)
{
	*(u8*)&sBuffer = bufferSel;
}

/*****************************************************/
/*													 */
/*	Convert pixel mode 1 to mode 0					 */
/*												     */
/*****************************************************/
/*u16 ConvertPixelMode1ToMode0(u8 pix)
{
	u8 pix0 = (pix & 0x80) >> 7;
	u8 pix2 = (pix & 0x40) >> 6;

	u8 pix4 = (pix & 0x20) >> 5;
	u8 pix6 = (pix & 0x10) >> 4;

	u8 pix1 = (pix & 0x08) >> 2;
	u8 pix3 = (pix & 0x04) >> 1;

	u8 pix5 = (pix & 0x02);
	u8 pix7 = (pix & 0x01) << 1;

	u8 pixel1Mode0 = cpct_px2byteM0(pix5 | pix4, pix7 | pix6);
	u8 pixel2Mode0 = cpct_px2byteM0(pix1 | pix0, pix3 | pix2);

	return pixel1Mode0 << 8 | pixel2Mode0;
}*/

/*****************************************************/
/*													 */
/*	Convert font mode 1 to mode 0					 */
/*												     */
/*****************************************************/
/*u8* ConvertToMode0(u8* font, u8 cx, u8 cy)
{
	static u16 sFontConv[FONTS_CX * FONTS_CY];
	
	u8 index = 0;
	u8 i;
	for (i = 0; i < cy*cx; i++)
		sFontConv[index++] = ConvertPixelMode1ToMode0(font[i]);

	return (u8*)sFontConv;
}*/

/*****************************************************/
/*													 */
/*	Display char							 		 */
/*												     */
/*****************************************************/
static void DisplayChar(u8 x, u8 y, char c)
{
	u8 cx, cy;
	u8 offset;
	u8* spriteFont;
	u8* spriteFonts;

	/** Number by default */
	u8 charaIndex = c - 48;
	
	/** Letters */
	if (c > 64 && c < 91)
		charaIndex = c - 65 + NUMBER_FONTS;
	else
	if (c == '.')
		charaIndex = FONT_DOT;
	else
	if (c == '-')
		charaIndex = FONT_MINUS;
	else
	if (c == ' ')
		charaIndex = FONT_SPACE;

	if (sInGameFont)
	{
		spriteFonts = (u8*)gFontGame_fonts_in_game;

		cx = FONTS_GAME_CX;
		cy = FONTS_GAME_CY;
		offset = OFFS_FONT_GAME;
	}
	else
	{
		spriteFonts = (u8*)gFontTitle_fonts;

		cx = FONTS_CX;
		cy = FONTS_CY;
		offset = OFFS_FONT_MENU;
	}

	spriteFont = spriteFonts + charaIndex*offset;

	if (sBuffer & (BACK_BUFFER | BOTH_BUFFER))
		DrawSpriteType(spriteFont, GetBackScreenPtr(x, y), cx, cy, SPRITE_OPAQUE);

	if (sBuffer & (SCREEN_BUFFER | BOTH_BUFFER))
		DrawSpriteType(spriteFont, GetScreenPtr(x, y), cx, cy, SPRITE_OPAQUE);
}

/*****************************************************/
/*													 */
/*	Display String							 		 */
/*												     */
/*****************************************************/
void PrintStr(u8 x, u8 y, const char* text)
{
    while(*text != '\0')
    {
		DisplayChar(x, y, *text);
		x += (sInGameFont ? FONTS_GAME_CX + 1 : FONTS_CX);
        text++;
    }
}

/*****************************************************/
/*													 */
/*	Display Int							 			 */
/*												     */
/*****************************************************/
void PrintInt(u8 x, u8 y, u16 val, u8 nbDigits)
{		
	/** [0, 65365] */
	char conv[6] = { '0', '\0'};
	u8 length = ConvertU16ToChar(val, conv);

	if (nbDigits > length)
	{
		char digits[6];

		u8 i;
		for (i = 0; i < nbDigits; i++)
			digits[i] = '0';

		digits[i] = '\0';

		for (i = 0; i < length; i++)
			digits[nbDigits - length + i] = conv[i];

		PrintStr(x, y, digits);
	}
	else
		PrintStr(x, y, conv);
}