//-----------------------------LICENSE NOTICE------------------------------------
//  This file is part of Bitume 
//  Copyright (C) 2017 Arnaud Bouche (Arnaud6128)
//
//  This program is free software: you can redistribute it and/or modify
//  it under the terms of the GNU Lesser General Public License as published by
//  the Free Software Foundation, either version 3 of the License, or
//  (at your option) any later version.
//
//  This program is distributed in the hope that it will be useful,
//  but WITHOUT ANY WARRANTY; without even the implied warranty of
//  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
//  GNU Lesser General Public License for more details.
//
//  You should have received a copy of the GNU Lesser General Public License
//  along with this program.  If not, see <http://www.gnu.org/licenses/>.
//-------------------------------------------------------------------------------

/*************** System includes ***************/
#include <cpctelera.h>
#include "wincpctelera_helper.h"

/*************** Game includes ****************/
#include "structures.h"
#include "video.h"
#include "sound.h"
#include "player.h"
#include "mines.h"
#include "missiles.h"
#include "explosions.h"
#include "objects.h"
#include "cars.h"
#include "gun.h"
#include "menus.h"
#include "toolkit.h"
#include "fontsHelper.h"
#include "keyboard.h"

CPCT_ABSOLUTE_LOCATION_AREA(0x8000);

/*************** External variables ***********/
extern u8 gCarSprite[PLAYER_CX * PLAYER_CY];
extern u8 gArmorColors[NB_ARMOR_COLORS];
extern u8 gBonusColor[NB_BONUS_COLOR];
extern const u8 gGamePalette[];
extern u8 gFillMode0, gFillMode1;
extern u8 gRoadColor;
extern SRoad gRoad;

/*************** Local defines ****************/
#define FONTS_CX		2

#define POS_CAR_LABEL	0
#define POS_CAR_X		130
#define POS_CAR_Y		50

#define POS_NAME_LABEL	POS_CAR_Y + 100
#define POS_NAME_Y		POS_NAME_LABEL + 20

#define POS_HORIZON		POS_NAME_LABEL - 45
#define	HORIZON_CX		SCREEN_CX_BYTE/2
#define	HORIZON_CY		36

#define POS_ARROW_Y		POS_CAR_Y + 20
#define ARROW_CX		4
#define ARROW_CY		7

#define POS_ROAD_Y_UP	POS_CAR_Y - 12
#define POS_ROAD_Y_DO	POS_ROAD_Y_UP + 50 - ROAD_INTRO_CY

#define ROAD_INTRO_CX	20
#define ROAD_INTRO_CY	10

#define FILL_Y			38
#define FILL_CY			53

#define BLACK_CAR		0
#define ASPHALT_CAR		1
#define PINK_CAR		2

#define DECOMPRESS_MEM	(0xA000)

/*************** Local Sprites ****************/
static const unsigned char G_car_asphalt[104] = {
	0xc0, 0x03, 0xc0, 0x00, 0x00, 0x40, 0xc0, 0x80, 0x04, 0xee, 0x08, 0x05, 0x50, 0x00, 0x40, 0x80,
	0x50, 0xf0, 0x50, 0xa0, 0x01, 0x3c, 0xff, 0xff, 0x01, 0x40, 0x50, 0xb4, 0x3c, 0x14, 0x3c, 0x3c,
	0x40, 0x05, 0x7d, 0xa2, 0x50, 0x3c, 0xa0, 0xb0, 0x1d, 0xf0, 0xf0, 0x7d, 0x08, 0x01, 0x28, 0x00,
	0x44, 0x50, 0xf5, 0xaa, 0xcc, 0x43, 0x0c, 0xa2, 0x78, 0x78, 0x0a, 0xf7, 0x1c, 0x03, 0xf0, 0x0c,
	0xaa, 0x07, 0x04, 0x0a, 0xd3, 0x03, 0xaa, 0xfa, 0x0a, 0x3a, 0x0a, 0xb4, 0x6e, 0x20, 0x51, 0x2d,
	0x15, 0x3c, 0x2d, 0x15, 0x45, 0x2b, 0x45, 0x41, 0x45, 0x57, 0x45, 0x6d, 0x04, 0x83, 0x50, 0x99,
	0x45, 0x04, 0xaf, 0x50, 0xc5, 0x40, 0x00, 0x20
};

static const unsigned char G_car_black[103] = {
	0xc0, 0x03, 0xc0, 0x00, 0x00, 0x40, 0xc0, 0x80, 0x04, 0xee, 0x08, 0x05, 0x50, 0x00, 0x40, 0x80,
	0x41, 0xc3, 0xa0, 0x00, 0xf0, 0x3c, 0x3c, 0x51, 0x15, 0x40, 0x00, 0x82, 0x01, 0x67, 0x02, 0x00,
	0xa2, 0x02, 0x25, 0x16, 0x06, 0x0a, 0x08, 0x28, 0xc3, 0xc6, 0x12, 0x41, 0x78, 0xc9, 0x0c, 0x15,
	0x69, 0x00, 0xc6, 0xaa, 0x06, 0x0e, 0x06, 0xc3, 0x82, 0x41, 0x00, 0x86, 0x0a, 0x82, 0x59, 0x86,
	0x2f, 0x41, 0x3c, 0x20, 0x0a, 0x96, 0x3c, 0x78, 0xfa, 0x0a, 0x2b, 0x0a, 0xf8, 0x20, 0x04, 0xcb,
	0x15, 0x36, 0xee, 0x15, 0x2b, 0x2a, 0x41, 0x2a, 0x57, 0x2a, 0x6d, 0x28, 0x83, 0x22, 0x82, 0x99,
	0x28, 0xaf, 0x22, 0x82, 0xc5, 0x00, 0x01
};

static const unsigned char G_car_pink[107] = {
	0xc0, 0x03, 0xc0, 0x00, 0x00, 0x40, 0xc0, 0x80, 0x04, 0xee, 0x08, 0x05, 0x50, 0x00, 0x40, 0x80,
	0x55, 0xbf, 0x30, 0x2a, 0x3f, 0x01, 0xfc, 0xff, 0xff, 0xa2, 0x00, 0x55, 0xfe, 0xfc, 0x54, 0xfc,
	0xfc, 0x15, 0x7e, 0x02, 0xfc, 0xfd, 0x08, 0x55, 0xfc, 0x2a, 0xc0, 0x1d, 0x3f, 0x3f, 0xfd, 0xa2,
	0x7c, 0x00, 0x37, 0x44, 0x88, 0x22, 0xcc, 0x0c, 0x8f, 0x25, 0x28, 0xbd, 0x44, 0x18, 0x03, 0x31,
	0x15, 0x3f, 0x0c, 0x55, 0xbd, 0x04, 0xed, 0x0a, 0x03, 0x15, 0x2b, 0xaa, 0x3f, 0x0a, 0x3a, 0xce,
	0x0a, 0x20, 0x3f, 0x42, 0xa8, 0x55, 0x15, 0x2b, 0x15, 0x41, 0x15, 0x57, 0x15, 0x6d, 0x14, 0x83,
	0x11, 0x41, 0x99, 0x14, 0xaf, 0x11, 0x41, 0xc5, 0x00, 0x00, 0x80
};

static const unsigned char G_arrow[28] = {
	0xc0, 0x80, 0xc0, 0xc0, 0xc0, 0x04, 0x40, 0xc0, 0x80, 0x1c, 0x08, 0xc0, 0x04, 0x78, 0x2c, 0x40,
	0x14, 0xa0, 0xb4, 0x40, 0x50, 0x40, 0x50, 0x40, 0x00, 0xc0, 0x80, 0x40
};

static const unsigned char G_road_intro[200] = {
	0xf0, 0xf0, 0xf0, 0xf0, 0xf0, 0xf0, 0xf0, 0xf0, 0xf0, 0xf0, 0x3c, 0x3c, 0x3c, 0x3c, 0x3c, 0x3c,
	0x3c, 0x3c, 0x3c, 0x3c, 0xf0, 0xf0, 0xf0, 0xf0, 0xf0, 0xf0, 0xf0, 0xf0, 0xf0, 0xf0, 0x3c, 0x3c,
	0x3c, 0x3c, 0x3c, 0x3c, 0x3c, 0x3c, 0x3c, 0x3c, 0xf0, 0xf0, 0xf0, 0xf0, 0xf0, 0xf0, 0xf0, 0xf0,
	0xf0, 0xf0, 0x3c, 0x3c, 0x3c, 0x3c, 0x3c, 0x3c, 0x3c, 0x3c, 0x3c, 0x3c, 0xf0, 0xf0, 0xf0, 0xf0,
	0xf0, 0xf0, 0xf0, 0xf0, 0xf0, 0xf0, 0x3c, 0x3c, 0x3c, 0x3c, 0x3c, 0x3c, 0x3c, 0x3c, 0x3c, 0x3c,
	0xf0, 0xf0, 0xf0, 0xf0, 0xf0, 0xf0, 0xf0, 0xf0, 0xf0, 0xf0, 0x3c, 0x3c, 0x3c, 0x3c, 0x3c, 0x3c,
	0x3c, 0x3c, 0x3c, 0x3c, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00,
	0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0xc0, 0xc0, 0xc0, 0xc0, 0xc0, 0xc0, 0xc0, 0xc0,
	0xc0, 0xc0, 0xc0, 0xc0, 0xc0, 0xc0, 0xc0, 0xc0, 0xc0, 0xc0, 0xc0, 0xc0, 0xc0, 0xc0, 0xc0, 0x84,
	0x0c, 0x0c, 0x0c, 0x0c, 0x0c, 0x0c, 0x0c, 0x0c, 0x0c, 0x0c, 0x0c, 0x0c, 0x48, 0xc0, 0xc0, 0xc0,
	0xc0, 0xc0, 0xc0, 0x84, 0x0c, 0x0c, 0x0c, 0x0c, 0x0c, 0x0c, 0x0c, 0x0c, 0x0c, 0x0c, 0x0c, 0x0c,
	0x48, 0xc0, 0xc0, 0xc0, 0xc0, 0xc0, 0xc0, 0xc0, 0xc0, 0xc0, 0xc0, 0xc0, 0xc0, 0xc0, 0xc0, 0xc0,
	0xc0, 0xc0, 0xc0, 0xc0, 0xc0, 0xc0, 0xc0, 0xc0
};

static const unsigned char G_horizon_0[228] = {
	0xfa, 0x82, 0x64, 0x00, 0xf5, 0x13, 0x10, 0x00, 0xfb, 0xff, 0x4a, 0x00, 0xf7, 0x82, 0x68, 0x27,
	0x12, 0x08, 0x4f, 0xcc, 0x00, 0x40, 0x4f, 0x88, 0x77, 0x8a, 0x0b, 0x1c, 0x11, 0xff, 0x00, 0x0e,
	0xe0, 0x27, 0xee, 0x00, 0xf7, 0x99, 0x11, 0x4a, 0x11, 0x19, 0xee, 0x7c, 0x33, 0x27, 0x1b, 0x9e,
	0x3d, 0x05, 0x0b, 0x11, 0xf6, 0x20, 0x0e, 0x88, 0x03, 0x88, 0x94, 0x27, 0xfb, 0x11, 0x9c, 0x27,
	0xcc, 0x11, 0x96, 0x27, 0xa9, 0x1b, 0x27, 0x00, 0x78, 0x27, 0x1e, 0x00, 0x66, 0x00, 0x9a, 0x4f,
	0xcb, 0x27, 0xee, 0x11, 0x0a, 0xee, 0x00, 0x27, 0xee, 0x06, 0x27, 0x6a, 0x00, 0x27, 0x15, 0x88,
	0xff, 0x11, 0x4f, 0xff, 0x07, 0x24, 0x4a, 0x27, 0xc3, 0x46, 0x1e, 0x00, 0x1e, 0x5b, 0x80, 0x41,
	0x00, 0x2a, 0x00, 0x40, 0x09, 0xa0, 0x27, 0x77, 0x4f, 0x21, 0x4b, 0x0f, 0xc1, 0x00, 0x08, 0x70,
	0xf0, 0x2a, 0x00, 0x87, 0x1b, 0x25, 0x27, 0x0c, 0x00, 0x26, 0x43, 0xf0, 0x42, 0x4f, 0x0e, 0x00,
	0x30, 0xe1, 0x21, 0xf2, 0x4f, 0x00, 0x10, 0xe1, 0x1f, 0xf2, 0x4f, 0x08, 0x00, 0xe1, 0x1d, 0xf9,
	0x4f, 0xc2, 0x98, 0x1d, 0x4f, 0x9c, 0x98, 0x21, 0xe9, 0x4f, 0xc2, 0x98, 0x1d, 0x4f, 0x9c, 0x98,
	0x21, 0xde, 0x4f, 0x25, 0xc2, 0x98, 0x1f, 0x4f, 0x9c, 0xbe, 0x21, 0xd9, 0x4f, 0xc2, 0xbe, 0x1e,
	0x4f, 0x9c, 0xbe, 0x21, 0xd9, 0x4f, 0xc2, 0xbe, 0x1d, 0x4f, 0x9c, 0x98, 0x21, 0xd9, 0x4f, 0xc2,
	0x98, 0x1e, 0x4f, 0x87, 0x42, 0x98, 0x78, 0x4f, 0x7c, 0x98, 0x26, 0x00, 0x61, 0x80, 0x98, 0x02,
	0x64, 0x00, 0x00, 0x02
};

static const unsigned char G_horizon_1[222] = {
	0xfa, 0x82, 0x64, 0x00, 0xf5, 0x13, 0x08, 0x00, 0x88, 0x77, 0xff, 0x20, 0x00, 0x08, 0x33, 0x33,
	0xff, 0xfe, 0x44, 0x27, 0x88, 0x33, 0x84, 0x90, 0x27, 0xfd, 0x4c, 0x27, 0xfe, 0x45, 0x00, 0x94,
	0x27, 0xee, 0x00, 0x04, 0x00, 0x9f, 0x27, 0x0d, 0x22, 0x00, 0x11, 0x04, 0x4f, 0x00, 0x22, 0x00,
	0x27, 0x27, 0x69, 0x20, 0xcc, 0x39, 0x27, 0x04, 0x67, 0x4f, 0x27, 0xff, 0x16, 0x0d, 0x27, 0x5d,
	0xed, 0x03, 0x27, 0x00, 0x3a, 0x27, 0xee, 0x54, 0x27, 0xb9, 0xdd, 0x27, 0x20, 0x87, 0x97, 0x13,
	0xf7, 0x06, 0x1d, 0x07, 0x17, 0x00, 0x69, 0x27, 0x98, 0xf1, 0x0c, 0x1d, 0x88, 0x00, 0xee, 0x2f,
	0x09, 0x44, 0xc7, 0x03, 0x90, 0x00, 0x20, 0x00, 0x4a, 0x00, 0x10, 0x87, 0xe0, 0x20, 0x29, 0xd2,
	0x4f, 0xf0, 0x88, 0x00, 0xe0, 0x01, 0x0f, 0x70, 0x00, 0x1e, 0xf0, 0x9c, 0x28, 0x00, 0x03, 0x86,
	0xd3, 0x27, 0x2d, 0xe2, 0x27, 0x00, 0xc0, 0x00, 0x07, 0x08, 0xac, 0x4f, 0x00, 0x80, 0x00, 0x82,
	0x3b, 0x4f, 0x00, 0x21, 0x00, 0x01, 0xf8, 0x4f, 0x78, 0xa6, 0x27, 0x4f, 0x87, 0xc2, 0xa6, 0x78,
	0x4f, 0x7c, 0xa6, 0x27, 0x4f, 0x87, 0xc2, 0xa6, 0x13, 0x4f, 0xb0, 0xa6, 0xe1, 0x29, 0xf8, 0x4f,
	0xf0, 0xd0, 0x87, 0x27, 0x4f, 0xd0, 0x08, 0x7e, 0x4f, 0x70, 0xd0, 0x87, 0xe3, 0x4f, 0xc2, 0xd0,
	0x1d, 0x4f, 0x9c, 0xa6, 0x21, 0xf8, 0x4f, 0xa0, 0xa6, 0x87, 0xe7, 0x4f, 0xa6, 0x08, 0x7e, 0x4f,
	0x70, 0xa6, 0x82, 0x3b, 0x4f, 0xa6, 0x02, 0x10, 0xf0, 0x09, 0xa0, 0x00, 0x00, 0x10
};

/*************** Interrupt handler initialize ******/
u16 gSeed;

void sInterrupInitHandler(void)
{
	static u8 sInterrupt = 0;
	cpct_setVideoMode(sInterrupt == 2 ? 0 : 1);

	if (++sInterrupt == 6)
	{
		sInterrupt = 0;
		cpct_scanKeyboard_if();
	}
}

/*****************************************************/
/*													 */
/*	Initialize sprites size							 */
/*												     */
/*****************************************************/
void InitSpritesSize()
{
	u8 i;
	for (i = 0; i < NB_MINES_DROP_MAX; i++)
	{
		gRoad._mines._mine[i]._spriteInfo._cx = MINE_CX;
		gRoad._mines._mine[i]._spriteInfo._cy = MINE_CY;
	}

	for (i = 0; i < NB_EXPLOSIONS_MAX; i++)
	{
		gRoad._explosions._explosion[i]._spriteInfo._cx = EXPLOSION_CX;
		gRoad._explosions._explosion[i]._spriteInfo._cy = EXPLOSION_CY;
	}

	for (i = 0; i < NB_MISS_LAUNCH_MAX; i++)
	{
		gRoad._missiles._missile[i]._spriteInfo._cx = MISSILE_CX;
		gRoad._missiles._missile[i]._spriteInfo._cy = MISSILE_CY;
	}

	for (i = 0; i < NB_OBJECTS_MAX; i++)
	{
		gRoad._objects._object[i]._spriteInfo._cx = OBJECT_CX;
		gRoad._objects._object[i]._spriteInfo._cy = OBJECT_CY;
	}
}

/*****************************************************/
/*													 */
/*	Initialize player car							 */
/*												     */
/*****************************************************/
void InitCarPlayer()
{
	SSpriteInfo* spriteInfo = &GET_PLAYER_OBJ._spriteInfo;
	spriteInfo->_sprite = gCarSprite;

	spriteInfo->_cx = PLAYER_CX;
	spriteInfo->_cy = PLAYER_CY;

	GET_PLAYER_CAR._gun = GUN_DOUBLE_RIGHT;
	SetGunSize(GET_PLAYER_CAR_PTR);
}

/*****************************************************/
/*													 */
/*	Initialize items color							 */
/*												     */
/*****************************************************/
void InitColors()	
{
	gArmorColors[0] = cpct_px2byteM0(0, 0);		// HW_BLACK
	gArmorColors[1] = cpct_px2byteM0(9, 9);		// HW_BLUE
	gArmorColors[2] = cpct_px2byteM0(5, 5);		// HW_RED
	gArmorColors[3] = cpct_px2byteM0(11, 11);	// HW_ORANGE
	gArmorColors[4] = cpct_px2byteM0(13, 13);	// HW_BRIGHT_YELLOW
	gArmorColors[5] = cpct_px2byteM0(12, 12);	// HW_BRIGHT_GREEN

	gBonusColor[0] = cpct_px2byteM0(12, 12);	// ROAD_BONUS_SPEED				- HW_BRIGHT_GREEN
	gBonusColor[1] = cpct_px2byteM0(13, 13);	// ROAD_BONUS_REPAIR			- HW_BRIGHT_YELLOW
	gBonusColor[2] = cpct_px2byteM0(3, 3);		// ROAD_BONUS_AMMO_ANTIARMOR	- HW_SKY_BLUE
	gBonusColor[3] = cpct_px2byteM0(2, 2);		// ROAD_BONUS_SHIELD			- HW_BRIGHT_WHITE
	gBonusColor[4] = cpct_px2byteM0(11, 11);	// ROAD_BONUS_MINE				- HW_ORANGE
	gBonusColor[5] = cpct_px2byteM0(6, 6);		// ROAD_BONUS_MISSILE			- HW_BRIGHT_RED

	gFillMode1 = cpct_px2byteM1(1, 1, 1, 1);		// HW_WHITE
	gFillMode0 = cpct_px2byteM0(1, 1);			// HW_WHITE
	gRoadColor = cpct_px2byteM0(1, 1);			// HW_WHITE
}

/*****************************************************/
/*													 */
/*	Draw string horizontal centered					 */
/*												     */
/*****************************************************/
void PrintStrCenter(char* label, u8 size, u8 y)
{
	PrintStr((SCREEN_CX_BYTE - FONTS_CX * (size - 1)) / 2, y, label);
}

/*****************************************************/
/*													 */
/*	Draw road intro									 */
/*												     */
/*****************************************************/
void DrawRoadIntro()
{
	u8 x;
	for (x = 0; x < SCREEN_CX_BYTE / ROAD_INTRO_CX; x++)
		DrawSpriteType(G_road_intro, GetScreenPtr(x*ROAD_INTRO_CX, POS_ROAD_Y_UP), ROAD_INTRO_CX, ROAD_INTRO_CY, SPRITE_OPAQUE);

	/** Copy Road Line to bottom */
	for (x = 0; x < ROAD_INTRO_CY; x++)
		cpct_memcpy(GetScreenPtr(0, POS_ROAD_Y_DO + ROAD_INTRO_CY - x - 1), GetScreenPtr(0, POS_ROAD_Y_UP + x), SCREEN_CX_BYTE);
}

/*****************************************************/
/*													 */
/*	Draw horizon									 */
/*												     */
/*****************************************************/
void DrawHorizon()
{
	u8* tempMem = WinCpcTelera_GetMemPtr((u16)DECOMPRESS_MEM);

	Decompress(G_horizon_0, tempMem);
	cpct_drawSprite(tempMem, GetScreenPtr(0, POS_HORIZON), HORIZON_CX, HORIZON_CY);

	Decompress(G_horizon_1, tempMem);
	cpct_drawSprite(tempMem, GetScreenPtr(40, POS_HORIZON), HORIZON_CX, HORIZON_CY);
}

/*****************************************************/
/*													 */
/*	Shift array to left								 */
/*												     */
/*****************************************************/
void ShiftLeftRoad(u8* val)
{
	u8 const temp[2] = { val[0], val[1] };
	cpct_memcpy(&val[0], &val[2], SCREEN_CX_BYTE - 2);
	cpct_memcpy(&val[SCREEN_CX_BYTE - 2], temp, 2);
}

/*****************************************************/
/*													 */
/*	Scroll road	by copy offset						 */
/*												     */
/*****************************************************/
void ScrollRoad()
{
	u8 i;

	cpct_waitVSYNC();
	for (i = 0; i < ROAD_INTRO_CY; i++)
		ShiftLeftRoad(GetScreenPtr(0, POS_ROAD_Y_UP + i));
	
	cpct_waitVSYNC();
	for (i = 0; i < ROAD_INTRO_CY; i++)
		ShiftLeftRoad(GetScreenPtr(0, POS_ROAD_Y_DO + i));
}

/*****************************************************/
/*													 */
/*	Select player car								 */
/*												     */
/*****************************************************/
void SelectCar()
{
	const u8 posSelect[] = { 9, 34, 58 };
	u8 key = 0, prevKey = 0;
	u8 carSelected = 1;

	static const char labelDriver[] = "WELCOME DRIVER";
	static const char labelCar[] = "CHOOSE YOUR CAR";

	FillWidthScreen(SCREEN_BUFFER, 0, SCREEN_CY_BYTE, gFillMode1);

	/** Fill Road background*/
	FillWidthScreen(SCREEN_BUFFER, FILL_Y, FILL_CY, gFillMode0);

	cpct_setPalette(gGamePalette, 16);
	cpct_setBorder(HW_WHITE);

	PrintStrCenter(labelDriver, sizeof(labelDriver), POS_CAR_LABEL);
	PrintStrCenter(labelCar, sizeof(labelCar), POS_CAR_LABEL + 15);

	/** Draw black car */
	Decompress(G_car_black, gCarSprite);
	cpct_drawSprite(gCarSprite, GetScreenPtr(posSelect[0], POS_CAR_Y), PLAYER_CX, PLAYER_CY);

	/** Draw asphalt car */
	Decompress(G_car_asphalt, gCarSprite);
	cpct_drawSprite(gCarSprite, GetScreenPtr(posSelect[1], POS_CAR_Y), PLAYER_CX, PLAYER_CY);

	/** Draw pink car */
	Decompress(G_car_pink, gCarSprite);
	cpct_drawSprite(gCarSprite, GetScreenPtr(posSelect[2], POS_CAR_Y), PLAYER_CX, PLAYER_CY);

	/** Draw road */
	DrawRoadIntro();

	do 
	{
		u8 pos = posSelect[carSelected] + PLAYER_CX / 2 - ARROW_CX / 2 + 1;
		cpct_waitVSYNC();
		cpct_drawSprite(G_arrow, GetScreenPtr(pos, POS_ARROW_Y), ARROW_CX, ARROW_CY);

		key = ReadKeyboardInput();

		if (prevKey != key)
		{
			if (key == VKEY_LEFT && carSelected > 0)
			{
				FillWidthScreen(SCREEN_BUFFER, POS_ARROW_Y, ARROW_CY, gFillMode0);
				carSelected--;
			}
			else if (key == VKEY_RIGHT && carSelected < 2)
			{
				FillWidthScreen(SCREEN_BUFFER, POS_ARROW_Y, ARROW_CY, gFillMode0);
				carSelected++;
			}

			prevKey = key;
		}
		
		gSeed++;
		ScrollRoad();
	}
	while(key != VKEY_ACTION);

	switch (carSelected)
	{
		case BLACK_CAR:		Decompress(G_car_black, gCarSprite);
			break;
		case ASPHALT_CAR:	Decompress(G_car_asphalt, gCarSprite);
			break;
		case PINK_CAR:		Decompress(G_car_pink, gCarSprite);
			break;
	}
}

/*****************************************************/
/*													 */
/*	Get character from cpckey entered				 */
/*												     */
/*****************************************************/
char GetASCII(cpct_keyID cpcKey)
{
	typedef struct TCpcChara
	{
		cpct_keyID _cpcKey;
		char _ascii;
	} SCpcChara;

	static const SCpcChara chara[] =
	{
		{ Key_A, 'A' },
		{ Key_B, 'B' },
		{ Key_C, 'C' },
		{ Key_D, 'D' },
		{ Key_E, 'E' },
		{ Key_F, 'F' },
		{ Key_G, 'G' },
		{ Key_H, 'H' },
		{ Key_I, 'I' },
		{ Key_J, 'J' },
		{ Key_K, 'K' },
		{ Key_L, 'L' },
		{ Key_M, 'M' },
		{ Key_N, 'N' },
		{ Key_O, 'O' },
		{ Key_P, 'P' },
		{ Key_Q, 'Q' },
		{ Key_R, 'R' },
		{ Key_S, 'S' },
		{ Key_T, 'T' },
		{ Key_U, 'U' },
		{ Key_V, 'V' },
		{ Key_W, 'W' },
		{ Key_X, 'X' },
		{ Key_Y, 'Y' },
		{ Key_Z, 'Z' },
		{ Key_Dot, '.' },
		{ Key_Space, ' ' },
	};

	{
		u8 i;
		for (i = 0; i < sizeof(chara) / sizeof(SCpcChara); i++)
		{
			if (chara[i]._cpcKey == cpcKey)
				return chara[i]._ascii;
		}
	}
	return 0;
}


/*****************************************************/
/*													 */
/*	Cheat mode										 */
/*												     */
/*****************************************************/
void EnterCheatMode()
{
	static const char cheatCode[] = "MAD MAX";
	static const char cheatMode[] = "CHEAT MODE";
	
	u8 i;
	for (i = 0; i < sizeof(cheatCode); i++)
	{
		if (cheatCode[i] != GET_PLAYER._name[i])
		{
			GET_PLAYER._cheatMode = FALSE;
			return;
		}
	}
	GET_PLAYER._cheatMode = TRUE;

	PrintStrCenter(cheatMode, sizeof(cheatMode), POS_NAME_Y + 20);

	while (!GetKeyPressed());
}

/*****************************************************/
/*													 */
/*	Enter name										 */
/*												     */
/*****************************************************/
void EnterName()
{ 
	cpct_keyID key;
	u8 letterEnteredIndex;

	static const char nameDefault[] = "PHANTIS";
	static const char labelName[] = "ENTER YOUR NAME";
	PrintStrCenter(labelName, sizeof(labelName), POS_NAME_LABEL);

	cpct_memset(GET_PLAYER._name, '-', NAME_LENGTH-1);
	cpct_memcpy(GET_PLAYER._name, nameDefault, sizeof(nameDefault) - 1);	
	letterEnteredIndex = sizeof(nameDefault) - 1;

	PrintStrCenter(GET_PLAYER._name, sizeof(GET_PLAYER._name), POS_NAME_Y);

	/** Clear action key */
	GetKeyPressed();
	while (TRUE)
	{
		key = GetKeyPressed();
		while (cpct_isKeyPressed(key));

		if (key == Key_Enter || key == Key_Return)
			break;

		if ((key == Key_BackSlash || key == Key_Del) && letterEnteredIndex > 0)
		{
			GET_PLAYER._name[--letterEnteredIndex] = '-';
		}
		else if (letterEnteredIndex < NAME_LENGTH - 1)
		{
			char ascii = GetASCII(key);
			if (ascii != 0)
			{
				GET_PLAYER._name[letterEnteredIndex++] = ascii;
			}
		}

		PrintStrCenter(GET_PLAYER._name, sizeof(GET_PLAYER._name), POS_NAME_Y);
	}

	if (letterEnteredIndex == 0)
	{
		cpct_memcpy(GET_PLAYER._name, nameDefault, sizeof(nameDefault) - 1);
		letterEnteredIndex = sizeof(nameDefault) - 1;
	}

	GET_PLAYER._name[letterEnteredIndex] = '\0';

	EnterCheatMode();
}

/*****************************************************/
/*													 */
/*	Initialize										 */
/*												     */
/*****************************************************/
void Initialize()
{
	cpct_disableFirmware();
	cpct_setVideoMode(1);
	SetAllPaletteBlack(),
	
	SetDisplay(VIDEO_MEM);
	SetBufferFont(SCREEN_BUFFER);

	InitColors();
	InitSpritesSize();
	InitCarPlayer();
	
	SetInterruptHandler(sInterrupInitHandler);

	SelectCar();

	DrawHorizon();
	EnterName();

	cpct_setSeed_glfsr16(gSeed);
	InitMenu(FALSE);
}

CPCT_RELOCATABLE_AREA();
