//-----------------------------LICENSE NOTICE------------------------------------
//  This file is part of Bitume 
//  Copyright (C) 2017 Arnaud Bouche (Arnaud6128)
//
//  This program is free software: you can redistribute it and/or modify
//  it under the terms of the GNU Lesser General Public License as published by
//  the Free Software Foundation, either version 3 of the License, or
//  (at your option) any later version.
//
//  This program is distributed in the hope that it will be useful,
//  but WITHOUT ANY WARRANTY; without even the implied warranty of
//  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
//  GNU Lesser General Public License for more details.
//
//  You should have received a copy of the GNU Lesser General Public License
//  along with this program.  If not, see <http://www.gnu.org/licenses/>.
//-------------------------------------------------------------------------------

/*************** System includes ***************/
#include <cpctelera.h>
#include "wincpctelera_helper.h"

/*************** Game includes ****************/
#include "structures.h"

/*************** Static variables *************/
static BOOL sKeySelectWeaponRelease;

/*************** Global variables *************/
const cpct_keyID gKeys[NB_KEY_FOR] = { Key_CursorUp, Key_CursorDown, Key_CursorLeft, Key_CursorRight, Key_Space, Key_Copy };

/*****************************************************/
/*													 */
/*	Get current key presses							 */
/*												     */
/*****************************************************/
cpct_keyID GetKeyPressed() 
{	
	/** From Ronaldo in cpcwiki forum */
	u8* keyStatus = cpct_keyboardStatusBuffer;
	u8 i;

	while (!cpct_isAnyKeyPressed());

	for (i = 0; i < 10; i++)
	{
		cpct_keyID keypressed = *keyStatus++ ^ 0xFF;
		if (keypressed)
			return (keypressed << 8) + i; 
	}

	return 0;
}

/*****************************************************/
/*													 */
/*	Redefine a action key							 */
/*												     */
/*****************************************************/
void RedefineKey(u8 idKey) 
{
	cpct_keyID key = GetKeyPressed();
	cpct_keyID* defKey = (cpct_keyID*)&gKeys[idKey];
	
	while (cpct_isKeyPressed(key));

	*defKey = key;
}

/*****************************************************/
/*													 */
/*	Read input										 */
/*												     */
/*****************************************************/
u8 ReadKeyboardInput()
{
	/** No key */
	u8 key = VKEY_NOTHING;

	/** Escape */
	if (cpct_isKeyPressed(Key_Esc))
		return VKEY_ESCAPE;

	/** Right */
	if (cpct_isKeyPressed(gKeys[KEY_FOR_RIGHT]) || cpct_isKeyPressed(Joy0_Right))
		key = VKEY_RIGHT;
	else
		/** Left */
		if (cpct_isKeyPressed(gKeys[KEY_FOR_LEFT]) || cpct_isKeyPressed(Joy0_Left))
			key = VKEY_LEFT;

	/** UP */
	if (cpct_isKeyPressed(gKeys[KEY_FOR_UP]) || cpct_isKeyPressed(Joy0_Up))
		key |= VKEY_UP;
	else
		/** Down */
		if (cpct_isKeyPressed(gKeys[KEY_FOR_DOWN]) || cpct_isKeyPressed(Joy0_Down))
			key |= VKEY_DOWN;

	/** Space */
	if (cpct_isKeyPressed(gKeys[KEY_FOR_FIRE]) || cpct_isKeyPressed(Joy0_Fire1))
		key |= VKEY_ACTION;
	else
	{
		/** Copy */
		if (cpct_isKeyPressed(gKeys[KEY_FOR_WEAPON]) || cpct_isKeyPressed(Joy0_Fire2))
		{
			if (sKeySelectWeaponRelease)
			{
				key |= VKEY_SELECTION;
				sKeySelectWeaponRelease = FALSE;
			}
		}
		/** NOT Copy */
		else
			if (!cpct_isKeyPressed(gKeys[KEY_FOR_WEAPON]))
				sKeySelectWeaponRelease = TRUE;
	}
	return key;
}