//-----------------------------LICENSE NOTICE------------------------------------
//  This file is part of Bitume 
//  Copyright (C) 2017 Arnaud Bouche (Arnaud6128)
//
//  This program is free software: you can redistribute it and/or modify
//  it under the terms of the GNU Lesser General Public License as published by
//  the Free Software Foundation, either version 3 of the License, or
//  (at your option) any later version.
//
//  This program is distributed in the hope that it will be useful,
//  but WITHOUT ANY WARRANTY; without even the implied warranty of
//  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
//  GNU Lesser General Public License for more details.
//
//  You should have received a copy of the GNU Lesser General Public License
//  along with this program.  If not, see <http://www.gnu.org/licenses/>.
//-------------------------------------------------------------------------------

/*************** System includes ***************/
#include "wincpctelera_helper.h"

/*************** Game includes ****************/
#include "structures.h"
#include "toolkit.h"
#include "video.h"
#include "sound.h"
#include "player.h"
#include "road.h"
#include "cars.h"
#include "mines.h"
#include "explosions.h"

/*************** External variables ***********/
extern SRoad gRoad;

/*************** Local defines ****************/
#define SPRITE_MINE			0
#define SPRITE_NAILS		2

#define TIMER_MINE			5
#define MINE_RELOAD_TIME	12

#define NB_MINE_FIELD		2

/*****************************************************/
/*													 */
/*	Get sprite mine according type					 */
/*												     */
/*****************************************************/
u8 GetSpriteMine(u8 typeMine)
{
	return (typeMine == MINE) ? SPRITE_MINE : SPRITE_NAILS;
}

/*****************************************************/
/*													 */
/*	Add Mine										 */
/*												     */
/*****************************************************/
void AddMine(u8 typeMine, u8 posX, u8 posY, u8 maxMine)
{	
	if (gRoad._mines._nbMinesDrop < maxMine)
	{
		SRoadObject* newMine = &gRoad._mines._mine[gRoad._mines._nbMinesDrop++];
		newMine->_counter = Rand();
		newMine->_data = typeMine;
		newMine->_spriteInfo._sprite = gMines_tileset[GetSpriteMine(typeMine)];

		SetCoord(newMine, posX, posY);
	}
}

/*****************************************************/
/*													 */
/*	Draw Mines										 */
/*												     */
/*****************************************************/
void DrawMines()
{
	u8* nbMines = &gRoad._mines._nbMinesDrop;
	if (*nbMines != 0)
	{
		SRoadObject* itMine = gRoad._mines._mine;

		u8 i;
		for (i = 0; i < *nbMines; i++)
		{
			SSpriteInfo* sprite = &itMine->_spriteInfo;
			if (itMine->_counter > TIME_TO_DEL)
			{
				SCar* collidedCar = TestCarCollision(sprite);
				sprite->_x -= SPEED_SCROLL;

				if (collidedCar != NULL)
				{
					CollideRoadObject(itMine, collidedCar);
					if (itMine->_data == MINE)
						AddExplosion(sprite->_x-2, sprite->_y);
				}
				else
				{
					if (GetPosRightX(sprite) < 0)
						itMine->_counter = IMMEDIATE_DEL;
					else
						DrawRoadObject(itMine, TIMER_MINE, gMines_tileset, GetSpriteMine(itMine->_data));
				}
			}
			else
			{
				itMine->_counter--;
				ClearScreenZone(sprite->_x - 1, sprite->_y, MINE_CX * 2, MINE_CY, BACK_BUFFER);
			}

			itMine++;
		}

		DeleteRoadObjects(gRoad._mines._mine, nbMines);
	}
}

/*****************************************************/
/*													 */
/*	Drop mine										 */
/*												     */
/*****************************************************/
void DropMine(u8 mineType, SCar *car)
{
	SSpriteInfo* sprite = &car->_roadObj._spriteInfo;
	AddMine(mineType, sprite->_x - MINE_CX - 1, sprite->_y + (sprite->_cy - MINE_CY)/2, (car == GET_PLAYER_CAR_PTR) ? NB_MINES_DROP_MAX : NB_MINES_DROP_MAX - 1);
	PlaySFX(FX_IMPACT, FX_IMPACT_CHANNEL);
}

/*****************************************************/
/*													 */
/*	Update mine counter								 */
/*												     */
/*****************************************************/
void UpdateMineCounter(u8 mineType, SCar *car)
{
	if (++car->_roadObj._counter == MINE_RELOAD_TIME)
	{
		car->_nbShoot--;
		car->_roadObj._counter = 0;
		DropMine(mineType, car);
	}
}

/*****************************************************/
/*													 */
/*	Add mine field									 */
/*												     */
/*****************************************************/
void AddMineField(u8 mine)
{
	u8 j;
	for (j = 0; j < NB_MINES_DROP_MAX-1; j++)
	{
		u8 posX = GetRandPositionX();
		u8 posY = GetRandPositionY();

		AddMine(mine == EN_MINE ? MINE : NAILS, posX, posY, NB_MINES_DROP_MAX - 1);
	}

	AddCar(START_UP_CAR, EN_MINE_FIELD);
}