//-----------------------------LICENSE NOTICE------------------------------------
//  This file is part of Bitume 
//  Copyright (C) 2017 Arnaud Bouche (Arnaud6128)
//
//  This program is free software: you can redistribute it and/or modify
//  it under the terms of the GNU Lesser General Public License as published by
//  the Free Software Foundation, either version 3 of the License, or
//  (at your option) any later version.
//
//  This program is distributed in the hope that it will be useful,
//  but WITHOUT ANY WARRANTY; without even the implied warranty of
//  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
//  GNU Lesser General Public License for more details.
//
//  You should have received a copy of the GNU Lesser General Public License
//  along with this program.  If not, see <http://www.gnu.org/licenses/>.
//-------------------------------------------------------------------------------

/*************** System includes ***************/
#include "wincpctelera_helper.h"

/*************** Game includes ****************/
#include "structures.h"
#include "sound.h"
#include "video.h"
#include "road.h"
#include "cars.h"
#include "player.h"
#include "missiles.h"
#include "explosions.h"

/*************** Local defines ****************/
#define SPRITE_MISSILE_R	0
#define SPRITE_MISSILE_L	2

#define TIMER_MISSILE		1
#define MISSILE_SPEED		4

#define	MISSILE_TO_RIGHT	0
#define	MISSILE_TO_LEFT		1

#define MISSILE_RELOAD_TIME	20

/*****************************************************/
/*													 */
/*	Add Missile										 */
/*												     */
/*****************************************************/
void LaunchMissile(SCar* car)
{
	static u8 sStart = 0;

	if (gRoad._missiles._nbMissilesLaunch < NB_MISS_LAUNCH_MAX)
	{
		u8 x, y;
		SRoadObject* newMissile = &gRoad._missiles._missile[gRoad._missiles._nbMissilesLaunch++];
		SSpriteInfo* sprite = &car->_roadObj._spriteInfo;
		newMissile->_counter = 0;
		
		if (car == GET_PLAYER_CAR_PTR)
		{
			newMissile->_data = MISSILE_TO_RIGHT;
			x = GetPosRightX(sprite) - 5;
			y = (sStart++ % 2) ? sprite->_y - 2 : GetPosRightY(sprite) - 4;
		}
		else
		{
			newMissile->_data = MISSILE_TO_LEFT;
			x = car->_roadObj._spriteInfo._x - 2;
			y = GetSpriteCenterY(sprite);
		}

		newMissile->_spriteInfo._x = x;
		newMissile->_spriteInfo._y = y;

		PlaySFX(FX_MISSILE, FX_MISSILE_CHANNEL);
	}
}

/*****************************************************/
/*													 */
/*	Clear Missile trail								 */
/*												     */
/*****************************************************/
void ClearMissileTrail(u8 dir, SSpriteInfo* missile, u8 buffer)
{
	ClearScreenZone((dir == MISSILE_TO_RIGHT) ? missile->_x - MISSILE_CX * 2 : GetPosRightX(missile), missile->_y, MISSILE_CX * 2, MISSILE_CY, buffer);
}

/*****************************************************/
/*													 */
/*	Draw Missiles									 */
/*												     */
/*****************************************************/
void DrawMissiles()
{
	u8* nbMissilesLaunch = &gRoad._missiles._nbMissilesLaunch;
	if (*nbMissilesLaunch != 0)
	{
		SRoadObject* itMissile = gRoad._missiles._missile;

		u8 i;
		for (i = 0; i < *nbMissilesLaunch; i++)
		{
			if (itMissile->_counter > TIME_TO_DEL)
			{
				SSpriteInfo* sprite = &itMissile->_spriteInfo;
				SCar* collidedCar = TestCarCollision(sprite);
				u8 dir = itMissile->_data;

				ClearMissileTrail(dir, sprite, BACK_BUFFER);
				sprite->_x += (dir == MISSILE_TO_RIGHT) ? MISSILE_SPEED : -MISSILE_SPEED;

				if (collidedCar != NULL)
				{
					/** Launcher not killed by its own missile */
					if ((collidedCar == GET_PLAYER_CAR_PTR && dir == MISSILE_TO_LEFT) || (collidedCar != GET_PLAYER_CAR_PTR && dir == MISSILE_TO_RIGHT))
					{
						CollideRoadObject(itMissile, collidedCar);
						ClearMissileTrail(dir, sprite, BOTH_BUFFER);
						AddExplosionObject(sprite);
						continue;
					}
				}
			
				if (sprite->_x > SCREEN_CX_BYTE + MISSILE_CX || GetPosRightX(sprite) + MISSILE_CX < 0)
					itMissile->_counter = TIME_TO_DEL;
				else
					DrawRoadObject(itMissile, TIMER_MISSILE, gMissile_tileset, (dir == MISSILE_TO_RIGHT) ? SPRITE_MISSILE_R : SPRITE_MISSILE_L);
			}
			else
				itMissile->_counter--;

			itMissile++;
		}

		DeleteRoadObjects(gRoad._missiles._missile, nbMissilesLaunch);
	}
}

/*****************************************************/
/*													 */
/*	Update mine counter								 */
/*												     */
/*****************************************************/
void UpdateMissileCounter(SCar *car)
{
	if (++car->_roadObj._counter == MISSILE_RELOAD_TIME)
	{
		car->_nbShoot--;
		car->_roadObj._counter = 0;
		LaunchMissile(car);
	}
}