//-----------------------------LICENSE NOTICE------------------------------------
//  This file is part of Bitume 
//  Copyright (C) 2017 Arnaud Bouche (Arnaud6128)
//
//  This program is free software: you can redistribute it and/or modify
//  it under the terms of the GNU Lesser General Public License as published by
//  the Free Software Foundation, either version 3 of the License, or
//  (at your option) any later version.
//
//  This program is distributed in the hope that it will be useful,
//  but WITHOUT ANY WARRANTY; without even the implied warranty of
//  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
//  GNU Lesser General Public License for more details.
//
//  You should have received a copy of the GNU Lesser General Public License
//  along with this program.  If not, see <http://www.gnu.org/licenses/>.
//-------------------------------------------------------------------------------

/*************** System includes ***************/
#include <stdio.h>
#include "wincpctelera_helper.h"

/*************** Game includes ****************/
#include "structures.h"
#include "video.h"
#include "objects.h"
#include "road.h"
#include "player.h"

/*************** External variables ***********/
extern SRoad gRoad;

/*************** Global variables ***********/
u8 gBonusColor[NB_BONUS_COLOR];

/*****************************************************/
/*													 */
/*	Add objects										 */
/*												     */
/*****************************************************/
void AddObject(u8 typeObject, u8 posX, u8 posY)
{
	if (gRoad._objects._nbObjects < NB_OBJECTS_MAX)
	{
		SRoadObject* newObj = &gRoad._objects._object[gRoad._objects._nbObjects++];
		newObj->_spriteInfo._sprite = (typeObject == ROAD_BONUS_LIFE) ? gRoadBonus_road_bonus_2 : (typeObject < ROAD_BONUS_TEMPORARY) ? gRoadBonus_road_bonus_0 : gRoadBonus_road_bonus_1;
		newObj->_counter = 0;
		newObj->_data = typeObject;

		SetCoord(newObj, posX, posY);
	}
}

/*****************************************************/
/*													 */
/*	Draw object										 */
/*												     */
/*****************************************************/
void DrawObject(SRoadObject* object)
{
	u8 x = object->_spriteInfo._x;

	if (x + OBJECT_CX > 0 && x < SCREEN_CX_BYTE - 1)
	{
		u8 typeObject = object->_data;
		u8* p = GetBackScreenPtr(x + 1, object->_spriteInfo._y + 2);

		DrawClippedRoadObjectOpaque(object);

		if (typeObject != ROAD_BONUS_LIFE)
			cpct_drawSolidBox(p, gBonusColor[ROAD_BONUS_NUM - typeObject], OBJECT_CX - 2, OBJECT_CY - 4);
	}
}

/*****************************************************/
/*													 */
/*	Draw objects									 */
/*												     */
/*****************************************************/
void DrawObjects()
{
	u8* nbObjects = &gRoad._objects._nbObjects;
	if (*nbObjects != 0)
	{
		SRoadObject* itObject = gRoad._objects._object;

		u8 i;
		for (i = 0; i < *nbObjects; i++)
		{
			SSpriteInfo* sprite = &itObject->_spriteInfo;
			if (itObject->_counter > TIME_TO_DEL)
			{
				SCar* collidedCar = TestCarCollision(sprite);
				itObject->_spriteInfo._x -= SPEED_SCROLL;

				if (collidedCar == GET_PLAYER_CAR_PTR)
				{
					CollideRoadObject(itObject, collidedCar);
					ClearScreenZone(sprite->_x - 1, sprite->_y, OBJECT_CX * 2, OBJECT_CY, BOTH_BUFFER);
				}
				else
				{
					if (sprite->_x + OBJECT_CX < 0)
						itObject->_counter = TIME_TO_DEL;
					else
						DrawObject(itObject);
				}
			}
			else
				itObject->_counter--;

			itObject++;
		}

		DeleteRoadObjects(gRoad._objects._object, nbObjects);
	}
}