//-----------------------------LICENSE NOTICE------------------------------------
//  This file is part of Bitume 
//  Copyright (C) 2017 Arnaud Bouche (Arnaud6128)
//
//  This program is free software: you can redistribute it and/or modify
//  it under the terms of the GNU Lesser General Public License as published by
//  the Free Software Foundation, either version 3 of the License, or
//  (at your option) any later version.
//
//  This program is distributed in the hope that it will be useful,
//  but WITHOUT ANY WARRANTY; without even the implied warranty of
//  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
//  GNU Lesser General Public License for more details.
//
//  You should have received a copy of the GNU Lesser General Public License
//  along with this program.  If not, see <http://www.gnu.org/licenses/>.
//-------------------------------------------------------------------------------

/*************** System includes ***************/
#include <cpctelera.h>
#include "wincpctelera_helper.h"

/*************** Game includes ****************/
#include "structures.h"
#include "game.h"
#include "sound.h"
#include "video.h"
#include "player.h"
#include "board.h"
#include "mines.h"
#include "cars.h"
#include "gun.h"
#include "objects.h"
#include "fontsHelper.h"
#include "toolkit.h"

/*************** Sprites includes *************/
#include "road.h"
#include ".\gen_src\mine.h"
#include ".\gen_src\flame.h"
#include ".\gen_src\line_border.h"

/*************** Compress Sprites includes ****/
#include ".\gen_src\asphalt.h"
#include ".\gen_src\road_borders.h"

/*************** Extern variables *************/
extern u8 gMode;

/*************** Local defines ***************/
#define REDRAW_SIZE_X 		2
#define REDRAW_SIZE_Y 		4

#define LINE_BORDER_CX		G_LINE_BORDER_W
#define LINE_BORDER_CY		G_LINE_BORDER_H

#define FLAMES_NB			(sizeof(gFlame_tileset)/sizeof(u8*))
#define FLAMES_CX			GFLAME_FLAME_00_W
#define FLAMES_CY			GFLAME_FLAME_00_H
		
#define ASPHALT_Y			110				
#define ASPHALT_CX			30
#define ASPHALT_CY			12

#define BONUS_RATE			100
#define SPEED_VALUE_BONUS	2
#define PROTECT_VALUE_BONUS	1
#define AMMO_BONUS			3

#define ValueInRange(value, min, max)	((value >= min) && (value <= max))

/*************** Game definition *************/
const SLevels gLevels = 
{{
	{
		{	// Level 0			
			{ { { START_QUARTER_1, EN_BARREL },		{ START_QUARTER_2, EN_BARREL },		{ START_QUARTER_3, EN_BARREL },		{ START_QUARTER_4, EN_BARREL } } },		// Round 0
			{ { { START_UP_CAR, EN_MBIKE_9_R },		{ START_DOWN_CAR, EN_MBIKE_9_R },	{ 0, EN_NO },						{ 0, EN_NO } } },						// Round 1
			{ { { START_UP_CAR, EN_MBIKE_9_N },		{ START_DOWN_CAR, EN_MBIKE_9_N },	{ 0, EN_NO },						{ 0, EN_NO } } },						// Round 2
			{ { { START_UP_CAR, EN_MBIKE_9_R },		{ START_DOWN_CAR, EN_MBIKE_9_R },	{ START_UP_CAR, EN_MBIKE_9 },		{ 0, EN_NO } } },						// Round 3
			{ { { START_UP_CAR, EN_CAR_3_UP },		{ START_DOWN_CAR, EN_CAR_3_DO },	{ START_MID_CAR, EN_MBIKE_9_N },	{ 0, EN_NO } } },						// Round 4
			{ { { START_QUARTER_1, EN_BARREL },		{ START_QUARTER_2, EN_BARREL },		{ START_QUARTER_3, EN_BARREL },		{ START_QUARTER_4, EN_BARREL } } },		// Round 5
			{ { { START_UP_CAR, EN_MBIKE_9_N },		{ START_QUARTER_2, EN_MBIKE_9_N },	{ START_QUARTER_3, EN_MBIKE_9 },	{ 0, EN_NO } } },						// Round 6
			{ { { START_UP_CAR, EN_CAR_3_UP },		{ START_DOWN_CAR, EN_CAR_3_DO },	{ 0, EN_NO },						{ 0, EN_NO } } },						// Round 7
			{ { { START_UP_CAR, EN_BARREL },		{ START_MID_CAR, EN_TRUCK_5 },		{ START_DOWN_CAR, EN_BARREL } ,		{ 0, EN_NO } } },						// Round 8
			{ { { 0, EN_NAILS },					{ 0, EN_NO },						{ 0, EN_NO },						{ 0, EN_NO } } }						// Round 9
		}
	},
	{
		{	// Level 1
			{ { { START_UP_CAR, EN_CAR_1 },			{ START_DOWN_CAR, EN_CAR_1 },		{ 0, EN_NO },						{ 0, EN_NO } } },						// Round 0
			{ { { START_UP_CAR, EN_CAR_1 },			{ START_DOWN_CAR, EN_CAR_1 },		{ START_MID_CAR, EN_MBIKE_9_N },	{ 0, EN_NO } } },						// Round 1
			{ { { START_MID_CAR, EN_BIKE_2 },		{ START_DOWN_CAR, EN_BIKE_2 },		{ START_RAND_CAR, EN_NO },			{ START_MID_CAR, EN_NO } } },			// Round 2
			{ { { START_UP_CAR, EN_MBIKE_9_R },		{ START_DOWN_CAR, EN_MBIKE_9_R },	{ 0, ROAD_BONUS_SHIELD },			{ 0, EN_NO } } },						// Round 3
			{ { { 0, EN_NO },						{ START_UP_CAR, EN_BIKE_2 },		{ START_DOWN_CAR, EN_BIKE_2 },		{ 0, EN_NO } } },						// Round 4
			{ { { START_UP_CAR, EN_CAR_3_UP },		{ START_DOWN_CAR, EN_CAR_3_DO },	{ 0, EN_NO },						{ 0, EN_NO } } },						// Round 5
			{ { { START_UP_CAR, EN_MBIKE_9_R },		{ START_DOWN_CAR, EN_MBIKE_9_R },	{ START_UP_CAR, EN_MBIKE_9 },		{ 0, EN_NO } } },						// Round 6
			{ { { START_MID_CAR, EN_BIKE_2 },		{ START_DOWN_CAR, EN_BIKE_2 },		{ START_RAND_CAR, EN_NO },			{ START_RAND_CAR, EN_NO } } },			// Round 7
			{ { { START_MID_CAR, EN_TRUCK_5 },		{ 0, ROAD_BONUS_AMMO_ANTIARMOR },	{ 0, EN_NO },						{ 0, EN_NO } } },						// Round 8
			{ { { 0, EN_MINE },						{ 0, ROAD_BONUS_LIFE },				{ 0, EN_NO },						{ 0, EN_NO } } }						// Round 9
		}
	},
	{
		{	// Level 2
			{ { { 0, ROAD_BONUS_REPAIR },			{ START_UP_CAR, EN_CAR_4 },			{ 0, EN_NO },						{ 0, EN_NO } } },						// Round 0
			{ { { START_UP_CAR, EN_MBIKE_9_R },		{ START_DOWN_CAR, EN_MBIKE_9_R },	{ 0, EN_NO },						{ 0, EN_NO } } },						// Round 1
			{ { { START_UP_CAR, EN_CAR_4 },			{ START_DOWN_CAR, EN_CAR_4 },		{ 0, EN_NO },						{ 0, EN_NO } } },						// Round 2
			{ { { START_UP_CAR, EN_BIKE_2 },		{ START_DOWN_CAR, EN_BIKE_2 },		{ 0, ROAD_BONUS_MINE },				{ 0, EN_NO } } },						// Round 3
			{ { { START_UP_CAR, EN_CAR_7 },			{ START_RAND_CAR, EN_BARREL },		{ START_RAND_CAR, EN_BARREL },		{ 0, EN_NO } } },						// Round 4
			{ { { START_DOWN_CAR, EN_CAR_7 },		{ 0, EN_NAILS },					{ 0, EN_NO },						{ 0, EN_NO } } },						// Round 5
			{ { { START_MID_CAR, EN_MBIKE_9 },		{ 0, EN_NO },						{ START_UP_CAR, EN_CAR_3_UP },		{ START_DOWN_CAR, EN_CAR_3_DO } } },	// Round 6
			{ { { START_DOWN_CAR, EN_TRUCK_5 },		{ 0, EN_NO },						{ 0, EN_NO },						{ 0, EN_NO } } },						// Round 7
			{ { { START_QUARTER_1, EN_BARREL },		{ START_QUARTER_2, EN_BARREL },		{ START_QUARTER_3, EN_BARREL },		{ START_QUARTER_4, EN_BARREL } } },		// Round 8
			{ { { START_DOWN_CAR, EN_TRUCK_5 },		{ START_UP_CAR, EN_TRUCK_5 },		{ 0, ROAD_BONUS_MISSILE },			{ 0, EN_NO } } }						// Round 9
		}
	},
	{
		{	// Level 3
			{ { { 0, EN_NO },						{ START_UP_CAR, EN_CAR_1 },			{ START_MID_CAR, EN_CAR_1 },		{ START_DOWN_CAR, EN_CAR_1 } } },		// Round 0			
			{ { { START_MID_CAR, EN_CAR_6 },		{ START_UP_CAR, EN_MBIKE_9_N },		{ START_DOWN_CAR, EN_MBIKE_9_N },	{ 0, EN_NO } } },						// Round 1
			{ { { START_QUARTER_1, EN_BARREL },		{ START_QUARTER_2, EN_BARREL },		{ START_QUARTER_3, EN_BARREL },		{ START_QUARTER_4, EN_BARREL } } },		// Round 2
			{ { { START_UP_CAR, EN_CAR_8 },			{ START_DOWN_CAR, EN_CAR_8 },		{ 0, ROAD_BONUS_REPAIR },			{ 0, EN_NO } } },						// Round 3
			{ { { START_MID_CAR, EN_TRUCK_5 },		{ 0, EN_NO },						{ 0, EN_NO },						{ 0, EN_NO } } },						// Round 4
			{ { { START_MID_CAR, EN_CAR_4 },		{ START_DOWN_CAR, EN_CAR_4 },		{ 0, EN_NO },						{ 0, EN_NO } } },						// Round 5
			{ { { START_UP_CAR, EN_CAR_6 },			{ START_DOWN_CAR, EN_CAR_6 },		{ 0, EN_NO },						{ 0, EN_NO } } },						// Round 6
			{ { { START_MID_CAR, EN_CAR_7 },		{ 0, EN_NO },						{ 0, EN_MINE },						{ 0, EN_NO } } },						// Round 7
			{ { { START_QUARTER_1, EN_BARREL },		{ START_QUARTER_2, EN_BARREL },		{ START_QUARTER_3, EN_BARREL },		{ START_QUARTER_4, EN_BARREL } } },		// Round 8
			{ { { START_MID_CAR, EN_CAR_11 },		{ START_UP_CAR, EN_CAR_1 },			{ START_DOWN_CAR, EN_CAR_1 } ,		{ 0, ROAD_BONUS_SHIELD } } }			// Round 9
		}
	},
	{
		{	// Level 4
			{ { { START_UP_CAR, EN_BIKE_2 },		{ START_MID_CAR, EN_CAR_6 },		{ START_DOWN_CAR, EN_BIKE_2 },		{ 0, EN_NO } } },						// Round 0			
			{ { { START_MID_CAR, EN_CAR_6 },		{ START_UP_CAR, EN_MBIKE_9_N },		{ START_DOWN_CAR, EN_MBIKE_9_N },	{ 0, EN_NO } } },						// Round 1
			{ { { 0, EN_NO },						{ START_UP_CAR, EN_CAR_7 },			{ 0, EN_NO },						{ 0, EN_NO } } },						// Round 2
			{ { { START_UP_CAR, EN_CAR_3_UP },		{ START_DOWN_CAR, EN_CAR_8 },		{ START_DOWN_CAR, EN_CAR_3_DO },	{ 0, EN_NO } } },						// Round 3
			{ { { START_MID_CAR, EN_TRUCK_5 },		{ 0, EN_NO },						{ 0, EN_NO },						{ 0, EN_NO } } },						// Round 4
			{ { { START_UP_CAR, EN_CAR_1 },			{ START_DOWN_CAR, EN_CAR_4 },		{ 0, EN_NO },						{ 0, EN_NO } } },						// Round 5
			{ { { START_UP_CAR, EN_CAR_6 },			{ START_DOWN_CAR, EN_CAR_1 },		{ START_UP_CAR, EN_CAR_1 },			{ 0, EN_NO } } },						// Round 6
			{ { { START_DOWN_CAR, EN_CAR_7 },		{ 0, EN_NO },						{ 0, EN_MINE },						{ 0, EN_NO } } },						// Round 7
			{ { { 0, EN_NO },						{ START_MID_CAR, EN_CAR_11 },		{ 0, EN_NO },						{ 0, EN_NO } } },						// Round 8
			{ { { START_UP_CAR, EN_CAR_8 },			{ START_MID_CAR, EN_CAR_11 },		{ START_DOWN_CAR, EN_CAR_1 },		{ 0, ROAD_BONUS_REPAIR } } }			// Round 9
		}
	}
}};

/*****************************************************/
/*													 */
/*	Initialize RoadScene							 */
/*												     */
/*****************************************************/
void InitRoadScene()
{
	InitBoard();
	InitRoad();

	/** Copy from backBuffer to  videoBuffer */
	cpct_memcpy(SCREEN_BUFF, CPCT_VMEM_START, SCREEN_SIZE);
	
	/** Update board */
	UpdateInfoWeapon();
	UpdateInfoAmmo();
	UpdateInfoBonus();
	UpdateInfoArmor();
	UpdateInfoLife();
	UpdateInfoScore();
}

/*****************************************************/
/*													 */
/*	InitRoad										 */
/*												     */
/*****************************************************/
void InitRoad()
{
	u8 x;
	u8* tempSpriteBuffer0 = GetOriginBackScreenPtr();
	
	/** Extract in backBuffer memory */
	Decompress(G_road_borders, tempSpriteBuffer0);

	/** Draw to videoBuffer road border */
	tempSpriteBuffer0 += gRoad._curLevel * ROAD_BORDER_CX * ROAD_BORDER_CY;
	for (x = 0; x < SCREEN_CX_BYTE / ROAD_BORDER_CX; x++)
		DrawSpriteType(tempSpriteBuffer0, GetScreenPtr(x*ROAD_BORDER_CX, ROAD_BORDER_UP), ROAD_BORDER_CX, ROAD_BORDER_CY, SPRITE_OPAQUE);

	for (x = 0; x < SCREEN_CX_BYTE / LINE_BORDER_CX; x++)
		DrawSpriteType(g_line_border, GetScreenPtr(x*LINE_BORDER_CX, ROAD_BORDER_UP + ROAD_BORDER_CY), LINE_BORDER_CX, LINE_BORDER_CY, SPRITE_OPAQUE);

	/** Copy Road Line to bottom */
	for (x = 0; x < ROAD_BORDER_CY + LINE_BORDER_CY; x++)
		cpct_memcpy(GetScreenPtr(0, ROAD_BORDER_DOWN + ROAD_BORDER_CY - x - 1), GetScreenPtr(0, ROAD_BORDER_UP + x), SCREEN_CX_BYTE);
}

/*****************************************************/
/*													 */
/*	Shift array to left								 */
/*												     */
/*****************************************************/
void ShiftLeft(u8 y)
{
	#define SPEED_OFFSET 1
	
	u8* front = GetScreenPtr(0, y);
	u8* back = GetBackScreenPtr(0, y);

    cpct_memcpy(back, &front[SPEED_OFFSET], SCREEN_CX_BYTE - SPEED_OFFSET);
	cpct_memcpy(&back[SCREEN_CX_BYTE - SPEED_OFFSET], front, SPEED_OFFSET);
}

/*****************************************************/
/*													 */
/*	Update road										 */
/*												     */
/*****************************************************/
void DrawRoadBorder()
{
	u8 i;
	for (i = 0; i < ROAD_BORDER_CY; i++)
	{
		ShiftLeft(ROAD_BORDER_UP + i);
		ShiftLeft(ROAD_BORDER_DOWN + i);
	}
}

/*****************************************************/
/*													 */
/*	Clear RoadObject zone on both screen buffer		 */
/*												     */
/*****************************************************/
void ClearRoadObject(SSpriteInfo* spriteInfo)
{
	ClearScreenZone(spriteInfo->_x, 
					spriteInfo->_y, 
					spriteInfo->_cx, 
					spriteInfo->_cy, 
					BOTH_BUFFER);
}

/*****************************************************/
/*													 */
/*	Draw RoadObject	background		 				*/
/*												     */
/*****************************************************/
void DrawRoadObjectBackground(const SSpriteInfo* spriteInfo)
{	
	if (spriteInfo->_buffIndex > 1)
	{
		u8 memIndex = !GetVMemIndex();

		i8 prevX = spriteInfo->_prevPos[memIndex]._x;
		u8 prevY = spriteInfo->_prevPos[memIndex]._y;

		i8 deltaX = prevX - spriteInfo->_x;
		i8 deltaY = prevY - spriteInfo->_y;

		if (deltaX != 0)
		{
			u8 cx = deltaX;
			i8 x = prevX;

			if (deltaX < 0)
				cx = -deltaX;
			else
			{
				cx = deltaX;
				x += spriteInfo->_cx - deltaX;
			}

			ClearScreenZone(x, prevY, cx, spriteInfo->_cy, BACK_BUFFER);
		}

		if (deltaY != 0)
		{
			u8 cy = deltaY;
			u8 y = prevY;

			if (deltaY < 0)
				cy = -deltaY;
			else
			{
				cy = deltaY;
				y += spriteInfo->_cy - deltaY;
			}
				
			ClearScreenZone(prevX, y, spriteInfo->_cx, cy, BACK_BUFFER);
		}
	} 
}

/*****************************************************/
/*													 */
/*	Get limit Y										 */
/*												     */
/*****************************************************/
u8 GetLimitY(u8 y, u8 cy)
{
	if (y < LIMIT_UP)
		return LIMIT_UP;

	if (y + cy > LIMIT_DOWN)
		return LIMIT_DOWN - cy;

	return y;
}

/*****************************************************/
/*													 */
/*	Get limit Sprite Y								 */
/*												     */
/*****************************************************/
u8 GetLimitSpriteY(SSpriteInfo* sprite)
{
	return GetLimitY(sprite->_y, sprite->_cy);
}

/*****************************************************/
/*													 */
/*	Set limit X										 */
/*												     */
/*****************************************************/
i8 GetLimitX(i8 x)
{
	if (x < LIMIT_LEFT)
		return LIMIT_LEFT;

	if ((u8)x > (u8)LIMIT_RIGHT)
		return LIMIT_RIGHT;

	return x;
}

/*****************************************************/
/*													 */
/*	Get random position Y on road					 */
/*												     */
/*****************************************************/
u8 GetRandPositionY()
{
	return LIMIT_UP + cpct_getRandom_glfsr16_u8() % (LIMIT_DOWN_PLAYER - LIMIT_UP);
}

/*****************************************************/
/*													 */
/*	Get random position Y on road					 */
/*												     */
/*****************************************************/
u8 GetRandPositionX()
{
	return SCREEN_CX_BYTE + cpct_getRandom_glfsr16_u8() % 45;
}

/*****************************************************/
/*													 */
/*	Test collision horizontal between two objects	 */
/*												     */
/*****************************************************/
BOOL TestHorizontalCollision(SSpriteInfo* A, SSpriteInfo* B)
{
	u8 APosCY = A->_y + A->_cy;
	u8 BPosCY = B->_y + B->_cy;

	return (ValueInRange(A->_y, B->_y, BPosCY) || ValueInRange(B->_y, A->_y, APosCY));
}

/*****************************************************/
/*													 */
/*	Test collision horizontal between two objects	 */
/*												     */
/*****************************************************/
BOOL TestVerticalCollision(SSpriteInfo* A, SSpriteInfo* B)
{
	i8 APosCX = A->_x + A->_cx;
	i8 BPosCX = B->_x + B->_cx;

	return (ValueInRange(A->_x, B->_x, BPosCX) || ValueInRange(B->_x, A->_x, APosCX));
}

/*****************************************************/
/*													 */
/*	Test collision between two objects				 */
/*												     */
/*****************************************************/
BOOL TestCollision(SSpriteInfo* A, SSpriteInfo* B)
{
	if (!TestVerticalCollision(A, B))
		return FALSE;

	return TestHorizontalCollision(A, B);
}

/*****************************************************/
/*													 */
/*	Test collision with car							*/
/*												     */
/*****************************************************/
SCar* TestCarCollision(SSpriteInfo* object)
{
	if (!TestCollision(object, &GET_PLAYER_OBJ._spriteInfo))
	{
		SCar* itCar = gRoad._cars._car;

		u8 indexCar;
		for (indexCar = 0; indexCar < gRoad._cars._nbCars; indexCar++)
		{
			if (TestCollision(object, &itCar->_roadObj._spriteInfo))
				return itCar;

			itCar++;
		}
		return NULL;
	}
	return GET_PLAYER_CAR_PTR;
}

/*****************************************************/
/*													 */
/*	Set road object coordinates						 */
/*												     */
/*****************************************************/
void SetCoord(SRoadObject* obj, i8 x, u8 y)
{
	obj->_spriteInfo._x = x;
	obj->_spriteInfo._y = y;
}

/*****************************************************/
/*													 */
/*	Get Sprite center								 */
/*												     */
/*****************************************************/
u8 GetSpriteCenterY(const SSpriteInfo* sprite)
{
	return sprite->_y + sprite->_cy / 2;
}

/*****************************************************/
/*													 */
/*	Get flames position								 */
/*												     */
/*****************************************************/
SPos* GetFlamesPos(SCar *car)
{
	static SPos sPosFlame;
	const SSpriteInfo* spriteInfo = &car->_roadObj._spriteInfo;
	sPosFlame._x = GetPosRightX(spriteInfo) - FLAMES_CX;
	sPosFlame._y = GetLimitY(GetSpriteCenterY(spriteInfo) - FLAMES_CY / 2, FLAMES_CY);

	return &sPosFlame;
}

/*****************************************************/
/*													 */
/*	Clear flames									 */
/*												     */
/*****************************************************/
void ClearFlamesBackground(SCar *car)
{
	SPos* posFlames = GetFlamesPos(car);
	ClearScreenZone(posFlames->_x, posFlames->_y, FLAMES_CX + SPEED_SCROLL * 2, FLAMES_CY, BACK_BUFFER);
}

/*****************************************************/
/*													 */
/*	Draw flames										 */
/*												     */
/*****************************************************/
void DrawFlames(SCar *car) 
{
	SPos* posFlames = GetFlamesPos(car);

	if (car->_flames == FLAMES_NB)
		car->_flames = 1;
		
	DrawClippedSprite(gFlame_tileset[car->_flames], posFlames->_x, posFlames->_y, FLAMES_CX, FLAMES_CY, SPRITE_MASKED, TRUE);
}

/*****************************************************/
/*													 */
/*	Draw explosive object							 */
/*												     */
/*****************************************************/
void DrawRoadObject(SRoadObject* object, u8 timer, u8** spritesState, u8 index)
{
	u8 counter = (u8)object->_counter;

	if (timer != counter)
		object->_spriteInfo._sprite = spritesState[index + (counter < timer ? 0 : 1)];

	if (object->_counter++ > timer * 2)
		object->_counter = 0;

	DrawClippedRoadObjectOpaque(object);
}

/*****************************************************/
/*													 */
/*	Delete road  object								 */
/*												     */
/*****************************************************/
void DeleteRoadObject(SRoadObject* objects, SRoadObject* objToDel, u8* nb)
{                                                               
	const SRoadObject* lastObject = &objects[--*nb];

	if (objToDel != lastObject)
		cpct_memcpy(objToDel, lastObject, sizeof(SRoadObject));
}

/*****************************************************/
/*													 */
/*	Delete road  objects							 */
/*												     */
/*****************************************************/
void DeleteRoadObjects(SRoadObject* roadObject, u8* nb)
{
	SRoadObject* itObj = roadObject;

	u8 i;
	for (i = 0; i < *nb; i++)
	{
		if (itObj->_counter <= TIME_TO_DEL - 1)
			DeleteRoadObject(roadObject, itObj, nb);
		
		itObj++;
	}
}

/*****************************************************/
/*													 */
/*	Get bonus type									 */
/*												     */
/*****************************************************/
u8 GetBonusType()
{
	return cpct_getRandom_glfsr16_u8() % NB_BONUS + ROAD_BONUS_LIFE;
}

/*****************************************************/
/*													 */
/*	Add bonus										 */
/*												     */
/*****************************************************/
void AddRoadBonus(u8 money)
{
	GET_PLAYER._bonus._enterBonus += money;
	if (GET_PLAYER._bonus._enterBonus > BONUS_RATE)
	{
		GET_PLAYER._bonus._enterBonus -= BONUS_RATE;
		AddObject(GetBonusType(), GetRandPositionX(), GetRandPositionY());
	}
}
/*****************************************************/
/*													 */
/*	Collide road object								 */
/*												     */
/*****************************************************/
void CollideRoadObject(SRoadObject* object, SCar* collidedCar)
{
	object->_counter = TIME_TO_DEL;
	if (!(collidedCar->_status & STATUS_FLAMES))
	{
		u8 objectId = object->_data;

		/** Permanent Bonus */
		if (objectId >= ROAD_BONUS_TEMPORARY)
			DefaultPlayer();

		switch (objectId)
		{
			/** Permanent Bonus */
			case ROAD_BONUS_SPEED:
				GET_PLAYER._car._speed = SPEED_VALUE_BONUS;
				break;
			case ROAD_BONUS_SHIELD:
				GET_PLAYER._car._protection = PROTECT_VALUE_BONUS;
				break;
			case ROAD_BONUS_REPAIR:
				break; 
			case ROAD_BONUS_AMMO_ANTIARMOR:			
				GET_PLAYER._car._gun = GUN_DOUBLE_RIGHT_ANTIARMOR;
				break;

			/** Bonus */
			case ROAD_BONUS_LIFE: 
				GET_PLAYER._life++;
				UpdateInfoLife();
				break;
			case ROAD_BONUS_MISSILE:
				GET_PLAYER._missiles += AMMO_BONUS;
				break;
			case ROAD_BONUS_MINE:
				GET_PLAYER._mines += AMMO_BONUS;
				break;

			/** Damages */
			case MINE:
				collidedCar->_armor = 0;
				return;
			default:	
				DamageCar(collidedCar, (objectId == NAILS) ? NAILS_DAMAGES : MISSILE_DAMAGES);
				return;
		}

		if (objectId >= ROAD_BONUS_LIFE)
		{
			PlaySFX(FX_BONUS, FX_BONUS_CHANNEL);

			if (objectId >= ROAD_BONUS_TEMPORARY)
			{
				GET_PLAYER._bonus._id = objectId - ROAD_BONUS_TEMPORARY;
				GET_PLAYER._bonus._counter = BONUS_COUNTER;
				UpdateInfoBonus();
				UpdateCounterBonus();
			}				
		}

		UpdateInfoAmmo();
	}
}

/*****************************************************/
/*													 */
/*	Clear road object								 */
/*												     */
/*****************************************************/
void ClearRoadObjects(SRoadObject* objects, u8 nb)
{
	SRoadObject* itObj = objects;

	u8 i;
	for (i = 0; i < nb; i++)
	{
		ClearScreenZone(itObj->_spriteInfo._x - itObj->_spriteInfo._cx / 2, itObj->_spriteInfo._y, itObj->_spriteInfo._cx * 2, itObj->_spriteInfo._cy, BACK_BUFFER);
		itObj++;
	}
}

/*****************************************************/
/*													 */
/*	Start Round and add entities					 */
/*												     */
/*****************************************************/
void StartRound()
{
	const SEntity* entities = (SEntity*)gLevels.level[gRoad._curLevel]._round[gRoad._curRound]._entity;
	u8 i;

	DeleteCars();
	
	for (i = 0; i < NB_ENTITY_MAX; i++)
	{
		u8 typeId = entities->_typeId;
		if (typeId != EN_NO)
		{
			if (typeId == EN_MINE || typeId == EN_NAILS)
				AddMineField(typeId);
			else 
			if (typeId >= EN_BONUS_LIFE)
				AddObject(typeId, GetRandPositionX(), GetRandPositionY());
			else 
				AddCar(entities->_y, entities->_typeId);
		}

		entities++;
	}

	if (GET_PLAYER._bonus._id == BONUS_SPEED)
		PlayerCarDecrBonus(DECR_BONUS_SPEED);
}

/*****************************************************/
/*													 */
/*	End game										 */
/*												     */
/*****************************************************/
void DrawEndGame()
{
	u8* tempVideoBuffer = GetScreenPtr(0, 100);
	gMode = BLACK_SCREEN;
	
	/** Extract in video memory */
	Decompress(G_asphalt, tempVideoBuffer);

	/** Blit vertical Mirror sprite */
	{
		u8* to = tempVideoBuffer + ASPHALT_CX*(ASPHALT_CY + ASPHALT_CY - 1);

		u8 i;
		for (i = 0; i < ASPHALT_CY; i++)
			cpct_memcpy(to - i*ASPHALT_CX, tempVideoBuffer + i*ASPHALT_CX, ASPHALT_CX);
	}

	/** Draw Asphalt Truck */
	cpct_drawSprite(tempVideoBuffer, GetBackScreenPtr((SCREEN_CX_BYTE - ASPHALT_CX) / 2, ASPHALT_Y), ASPHALT_CX, ASPHALT_CY * 2);

	/** Draw end message */
	FillWidthScreen(BOTH_BUFFER, 0, BOARD_CY + 1, 0);
	SetGameFont(FALSE);
	PrintStr((SCREEN_CX_BYTE - 9 * 2) / 2, 2, "WELL DONE");
	PrintStr((SCREEN_CX_BYTE - 18 * 2) / 2, 16, "THE CONVOY IS SAFE");

	/** Copy from backBuffer to videoBuffer */ 
	cpct_memcpy(GetOriginScreenPtr(), GetOriginBackScreenPtr(), SCREEN_SIZE);

	gMode = WIN;
}
