//-----------------------------LICENSE NOTICE------------------------------------
//  This file is part of Bitume 
//  Copyright (C) 2017 Arnaud Bouche (Arnaud6128)
//
//  This program is free software: you can redistribute it and/or modify
//  it under the terms of the GNU Lesser General Public License as published by
//  the Free Software Foundation, either version 3 of the License, or
//  (at your option) any later version.
//
//  This program is distributed in the hope that it will be useful,
//  but WITHOUT ANY WARRANTY; without even the implied warranty of
//  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
//  GNU Lesser General Public License for more details.
//
//  You should have received a copy of the GNU Lesser General Public License
//  along with this program.  If not, see <http://www.gnu.org/licenses/>.
//-------------------------------------------------------------------------------

#include "defines.h"

#define NB_IMPACTS_MAX			10
#define NB_MINES_DROP_MAX		10
#define NB_MISS_LAUNCH_MAX		6
#define NB_EXPLOSIONS_MAX		4
#define NB_CARS_MAX				4
#define NB_ENTITY_MAX			4
#define	NB_ROUND_MAX			10
#define	NB_LEVEL_MAX			5
#define	NB_OBJECTS_MAX			3
#define	NAME_LENGTH				10

/** Functions pointer */
typedef void (*TMove)(void *car);
typedef u8* (*TGetBuffer)(u8 x, u8 y);

typedef struct TPos
{
	i8 _x;
	u8 _y;
} SPos;

typedef struct TSpriteInfo
{
	i8 _x;
	u8 _y;
	u8 _cx, _cy;

	u8 _buffIndex;
	SPos _prevPos[2];
	
	const u8* _sprite;
} SSpriteInfo;

/** Game structures */
typedef struct TRoadObject
{
	u8 _data;
	i8 _counter;
	SSpriteInfo _spriteInfo;
} SRoadObject;

typedef struct TCar
{
	u8 _status;
	u8 _weapon;
	u8 _gun;
	u8 _flames;
	u8 _type;
	u8 _armor, _protection;
	u8 _waitTime;
	u8 _nbShoot;
	u8 _speed;

	TMove Move;

	SSpriteInfo _gunSprite;
	SRoadObject _roadObj;
} SCar;

typedef struct TMines
{
	u8 _nbMinesDrop;
	SRoadObject _mine[NB_MINES_DROP_MAX];
} SMines;

typedef struct TMissiles
{
	u8 _nbMissilesLaunch;
	SRoadObject _missile[NB_MISS_LAUNCH_MAX];
} SMissiles;

typedef struct TExplosions
{
	u8 _nbExplosions;
	SRoadObject _explosion[NB_EXPLOSIONS_MAX];
} SExplosions;

typedef struct TRoadObjects
{
	u8 _nbObjects;
	SRoadObject _object[NB_OBJECTS_MAX];
} SRoadObjects;

/** Level definition */
typedef struct TEntity
{	
	u8 _y;
	u8 _typeId;
} SEntity;

typedef struct TRound
{
	SEntity _entity[NB_ENTITY_MAX];
} SRound;

typedef struct TLevel
{
	SRound _round[NB_ROUND_MAX];
} SLevel;

typedef struct TLevels
{
	SLevel level[NB_LEVEL_MAX];
}SLevels;

typedef struct TCarGraphics
{
	u8 _cx;
	u8 _cy;
	u8* _sprite;
} SCarGraphics;

typedef struct TCarInfo
{
	const u8 _armorProtect;
	const u8 _weapon;
	const i8 _posWeaponY;
	const u8 _speedScore;
	const u8 _graphics;
	const TMove _move;
} SCarInfo;

typedef struct TCars
{
	u8 _nbCars;
	SCar _car[NB_CARS_MAX];
} SCars;

typedef struct TImpact
{
	u8 _x;
	u8 _y;
	i8 _time;
} SImpact;

typedef struct TImpacts
{
	u8 _nbImpacts;
	SImpact _impact[NB_IMPACTS_MAX];
} SImpacts;

typedef struct TBonus
{
	u8 _id;
	u8 _counter;
	u8 _enterBonus;
} SBonus;

typedef struct TPlayer
{
	BOOL _cheatMode;
	u8 _life;
	u8 _mineReload, _missileReload;
	u8 _weaponSelect;
	u8 _heat;
	u8 _mines;
	u8 _missiles;
	u16 _money;

	SBonus _bonus;
	SCar _car;

	char _name[NAME_LENGTH];
} SPlayer;

typedef struct TRoad
{
	u8 _curRound, _curLevel;
	SRoadObjects _objects;
	SCars _cars;
	SMines _mines;
	SMissiles _missiles;
	SExplosions _explosions;
	SPlayer _player;
} SRoad;
