//-----------------------------LICENSE NOTICE------------------------------------
//  This file is part of Bitume 
//  Copyright (C) 2017 Arnaud Bouche (Arnaud6128)
//
//  This program is free software: you can redistribute it and/or modify
//  it under the terms of the GNU Lesser General Public License as published by
//  the Free Software Foundation, either version 3 of the License, or
//  (at your option) any later version.
//
//  This program is distributed in the hope that it will be useful,
//  but WITHOUT ANY WARRANTY; without even the implied warranty of
//  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
//  GNU Lesser General Public License for more details.
//
//  You should have received a copy of the GNU Lesser General Public License
//  along with this program.  If not, see <http://www.gnu.org/licenses/>.
//-------------------------------------------------------------------------------

/*************** System includes ***************/
#include <cpctelera.h>

/*************** Game includes ****************/
#include "structures.h"
#include "video.h"

/*************** Extern functions **************/
#include "utils\ZX7Decrunch.h"

/*****************************************************/
/*													 */
/*	Convert u16 to string							 */
/*												     */
/*****************************************************/
u8 ConvertU16ToChar(u16 num, char* str)
{
	u16 n = num;
	u8 i, len = 0;

	while (n != 0)
	{
		len++;
		n /= 10;
	}

	for (i = 0; i < len; i++)
	{
		str[len - (i + 1)] = num % 10 + '0';
		num = num / 10;
	}

	return len;
}

/*****************************************************/
/*													 */
/*	Set interrupt handler				 			 */
/*												     */
/*****************************************************/
void SetInterruptHandler(TInterrupt interruptFunction)
{
	cpct_waitVSYNC();
#ifndef WINCPCTELERA
	__asm
	halt
	halt
	__endasm;
#endif
	cpct_waitVSYNC();

	cpct_setInterruptHandler(interruptFunction);
}

/*****************************************************/
/*													 */
/*	Fill screen										 */
/*												     */
/*****************************************************/
void FillScreen(u8 vMem, u8 x, u8 y, u8 cx, u8 cy, u8 value)
{
	if (vMem & (SCREEN_BUFFER | BOTH_BUFFER))
		cpct_drawSolidBox(GetScreenPtr(x, y), value, cx, cy);

	if (vMem & (BACK_BUFFER | BOTH_BUFFER))
		cpct_drawSolidBox(GetBackScreenPtr(x, y), value, cx, cy);
}

/*****************************************************/
/*													 */
/*	Fill Width screen								 */
/*												     */
/*****************************************************/
void FillWidthScreen(u8 vMem, u8 y, u8 cy, u8 value)
{
	u8 i;
	for (i = 0; i < 2; i++)
	{
		u8 x = (i == 0) ? 0 : SCREEN_CX_BYTE / 2;
		FillScreen(vMem, x, y, SCREEN_CX_BYTE / 2, cy, value);
	}
}

/*****************************************************/
/*													 */
/*	Get random number 0 to 7 range		 			 */
/*												     */
/*****************************************************/
u8 Rand()
{
	return (cpct_getRandom_glfsr16_u8() & 0x07);
}

/*****************************************************/
/*													 */
/*	Use current decompressor			 			 */
/*												     */
/*****************************************************/
void Decompress(const u8* input, u8* output)
{
	ZX7Decrunch(input, output);
}

/*****************************************************/
/*													 */
/* Decrement positive value				 			 */
/*												     */
/*****************************************************/
void DecrValue(u8* value, u8 offset)
{
	if (*value > offset)
		*value -= offset;
	else
		*value = 0;
}
